# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolib as FnGeolib
import PyFnGeolibProducers as FnGeolibProducers
import PyFnGeolib
import PyFnGeolibProducers
import PyUtilModule.WorkingSet
from PyUtilModule.WorkingSet import LocationStateMap as LocationStateMap, WorkingSet as WorkingSet
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class WorkingSetClient:
    _WorkingSetClient__DELEGATE_TO_CLIENT: ClassVar[list] = ...
    _WorkingSetClient__DISALLOWED_WHEN_WORKINGSET_SET: ClassVar[list] = ...
    def __init__(self, txn: PyFnGeolib.GeolibRuntimeTransaction, op: PyFnGeolib.GeolibRuntimeOp) -> None: ...
    def _WorkingSetClient__applyWorkingSetToClient(self): ...
    def _WorkingSetClient__locationStateChangedCallback(self, stateChanges: list[tuple], workingSet: WorkingSet, sender: object | None): ...
    def _WorkingSetClient__openIncludedWithChildrenLocations(self, locationPaths: Incomplete | None = ...): ...
    def eventsFiltered(self) -> bool: ...
    def forceAsyncCookLocation(self, locationPath: str): ...
    def getLocationEvents(self, maxEvents: Incomplete | None = ...) -> list[PyFnGeolib.LocationDataChange]: ...
    def getProducerAtLocation(self, locationPath: str, producerType: str = ...) -> PyFnGeolibProducers.GeometryProducer: ...
    def getWorkingSet(self) -> PyUtilModule.WorkingSet.WorkingSet | None: ...
    def removeWorkingSet(self, openLocations: Incomplete | None = ..., activeLocations: Incomplete | None = ...): ...
    def setEventsFiltered(self, eventsFiltered: bool): ...
    def setOp(self, transaction: PyFnGeolib.GeolibRuntimeTransaction, op: PyFnGeolib.GeolibRuntimeOp): ...
    def setWorkingSet(self, workingSet: WorkingSet): ...
    def updateFollowingPotentialOpChange(self): ...
    def __dir__(self) -> list[str]: ...
    def __getattr__(self, name): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyUtilModule.EnvUtils as EnvUtils
import Utils.EventModule as EventModule
import PyFnGeolib as FnGeolib
import GeoAPI as GeoAPI
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyUtilModule.RenderManager as RenderManager
import RenderingAPI as RenderingAPI
import Utils as Utils
from _typeshed import Incomplete
from typing import Set, Tuple

def CanPrintDebugOutput() -> bool: ...
def WriteRenderDebug(node: NodegraphAPI.Node, renderer, filename: Incomplete | None = ..., expandProcedural: bool = ..., openInEditor: bool = ..., customEditor: Incomplete | None = ..., log: Incomplete | None = ...): ...
def WriteRenderOutput(node: NodegraphAPI.Node, renderer, filename: Incomplete | None = ..., expandProcedural: bool = ..., openInEditor: bool = ..., customEditor: Incomplete | None = ..., log: Incomplete | None = ...): ...
def WriteRenderOutputForRenderMethod(renderMethodName, node: NodegraphAPI.Node, renderer, filename: Incomplete | None = ..., expandProcedural: bool = ..., log: Incomplete | None = ..., openInEditor: bool = ..., customEditor: Incomplete | None = ..., sceneGraphPath: Incomplete | None = ..., printToConsole: bool = ...): ...
def WriteSerializedOpTreeToDisk(node: NodegraphAPI.Node, applyRenderResolvers, log: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute
from typing import Set, Tuple

class AttrMap:
    def __init__(self, *args, **kwargs) -> None: ...
    def get(self, key: str) -> PyFnAttribute.Attribute: ...
    def getKeys(self) -> PyFnAttribute.StringAttribute: ...

class LookFile:
    def __init__(self, filePath: str, passName: str = ...) -> None: ...
    def getAttrs(self, locationName: str, rootId: str = ...) -> AttrMap: ...
    def getMaterial(self, materialName: str, asGlobal: bool = ...) -> PyFnAttribute.Attribute: ...
    def getMaterialType(self, materialName: str) -> str: ...
    def getMaterials(self) -> AttrMap: ...
    @classmethod
    def getPassNamesForLookFileAsset(cls, asset: str) -> PyFnAttribute.StringAttribute: ...
    def getPathsWithOverrides(self, rootId: str = ...) -> PyFnAttribute.StringAttribute: ...
    def getRootIdNames(self) -> PyFnAttribute.StringAttribute: ...
    def getRootIdType(self, rootId: str) -> str: ...
    def getRootOverrides(self) -> AttrMap: ...
    @classmethod
    def getSafePath(cls, asset: str, includeVersion: bool = ...) -> PyFnAttribute.StringAttribute: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import NodegraphAPI as NodegraphAPI
from UI4.FormMaster.ParameterPolicy import CreateParameterPolicy as CreateParameterPolicy
from _typeshed import Incomplete
from typing import Set, Tuple

def DocDump(names: Incomplete | None = ..., outputdir: Incomplete | None = ...): ...
def DoxyTemplate(name, outputdir): ...
def DumpHelp(policy, indent: int = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import DopeSheet as DopeSheet, DopeSheetTree as DopeSheetTree, KeyDrawingLayer as KeyDrawingLayer, KeyInteractionLayer as KeyInteractionLayer, KeyLayerStack as KeyLayerStack, PanAndScrollLayer as PanAndScrollLayer
from UI4.Tabs.DopeSheet.DopeSheet import DopeSheetPanel as DopeSheetPanel
from typing import Set, Tuple

PluginRegistry: list



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import Catalog as Catalog, CatalogItem as CatalogItem, CatalogUtils as CatalogUtils, Client as Client
from CatalogAPI.Catalog import CreateCatalogItem as CreateCatalogItem, DeleteCatalogItems as DeleteCatalogItems, DuplicateCatalogItem as DuplicateCatalogItem, FillDragDataFromItem as FillDragDataFromItem, FindCatalogItemBySequenceID as FindCatalogItemBySequenceID, FindUnlockedCatalogItemBySlot as FindUnlockedCatalogItemBySlot, GetCatalogItems as GetCatalogItems, GetCatalogThumbnailWidth as GetCatalogThumbnailWidth, GetFirstCatalogItem as GetFirstCatalogItem, GetItemFromDragData as GetItemFromDragData, GetLastCatalogItem as GetLastCatalogItem, GetNextCatalogItem as GetNextCatalogItem, GetNumCatalogItems as GetNumCatalogItems, GetPreviousCatalogItem as GetPreviousCatalogItem, MoveCatalogItem as MoveCatalogItem, MoveCatalogItemToTop as MoveCatalogItemToTop, SetCatalogThumbnailWidth as SetCatalogThumbnailWidth
from CatalogAPI.CatalogUtils import ExportCatalogToDisk as ExportCatalogToDisk, GetCommonResolutionNameFromItems as GetCommonResolutionNameFromItems, GetFirstFrame as GetFirstFrame, GetMemoryUsageMB as GetMemoryUsageMB, GetNearestTimeInSequence as GetNearestTimeInSequence, GetSequenceCreationTime as GetSequenceCreationTime, GetSequenceObj as GetSequenceObj, GetUniqueImageNumberInDirectory as GetUniqueImageNumberInDirectory, ImportImageSequence as ImportImageSequence, IsPersistantCatalogAllowed as IsPersistantCatalogAllowed, ValidatePersistantDirectory as ValidatePersistantDirectory
from CatalogAPI.Client import ApplyCatalogItemProjectSettings as ApplyCatalogItemProjectSettings, AutoViewCatalogItem as AutoViewCatalogItem, CreateCatalogClientKey as CreateCatalogClientKey, DeleteCatalogClientKey as DeleteCatalogClientKey, GetAllowedViewSlots as GetAllowedViewSlots, GetClientCatalogItems as GetClientCatalogItems, GetClientKeys as GetClientKeys, GetClientViewSlots as GetClientViewSlots, GetCurrentlyViewedSlots as GetCurrentlyViewedSlots, GetPrimaryClientKey as GetPrimaryClientKey, GetRecommendedRenderSlots as GetRecommendedRenderSlots, GetUnassignedSlot as GetUnassignedSlot, IsClientPinned as IsClientPinned, SetClientCatalogItem as SetClientCatalogItem, SetClientPinned as SetClientPinned, SetClientViewSlot as SetClientViewSlot, SwapClientViewSlots as SwapClientViewSlots
from typing import Set, Tuple

RENDER_STATE_CANCELLED: str
RENDER_STATE_COMPLETED: str
RENDER_STATE_ERROR: str
RENDER_STATE_IN_PROGRESS: str
RENDER_STATE_NONE: str
kPrimaryView: int
kSecondaryView: int



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import Set, Tuple

def Critical(title, text, acceptText: Incomplete | None = ..., cancelText: Incomplete | None = ..., parent: Incomplete | None = ..., detailedText: Incomplete | None = ...): ...
def Information(title, text, acceptText: Incomplete | None = ..., cancelText: Incomplete | None = ..., parent: Incomplete | None = ..., detailedText: Incomplete | None = ...): ...
def Question(title, text, acceptText: Incomplete | None = ..., cancelText: Incomplete | None = ..., parent: Incomplete | None = ..., detailedText: Incomplete | None = ...): ...
def Tristate(title, text, yesText: Incomplete | None = ..., noText: Incomplete | None = ..., cancelText: Incomplete | None = ..., icon: PyQt5.QtWidgets.QMessageBox.Icon = ..., Parent: Incomplete | None = ..., detailedText: Incomplete | None = ...): ...
def Warning(title, text, acceptText: Incomplete | None = ..., cancelText: Incomplete | None = ..., parent: Incomplete | None = ..., detailedText: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import SafeIdentifier as SafeIdentifier, UniqueName as UniqueName
from Naming.SafeIdentifier import GetSafeIdentifier as GetSafeIdentifier
from Naming.UniqueName import GetUniqueName as GetUniqueName
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.IconManager as IconManager
import QT4FormWidgets.InputWidgets as InputWidgets
import NodegraphAPI as NodegraphAPI
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtCore
import PyXmlIO as PyXmlIO
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import Nodes3DAPI.ScenegraphManager as ScenegraphManager
import UI4.Widgets.SortablePanel
import Utils as Utils
from UI4.FormMaster.Editors.CelEditor import PathsStatementPanel as PathsStatementPanel
from UI4.FormMaster.Editors.ScenegraphLocationArray import ScenegraphLocationPanel as ScenegraphLocationPanel
from UI4.KatanaPrefs.KatanaPrefsObject import Prefs as Prefs
from UI4.Widgets.SortablePanel import SortablePanelFormWidget as SortablePanelFormWidget
from _typeshed import Incomplete
from typing import Set, Tuple

class LocationEditorFormWidget(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _LocationEditorFormWidget__addMenuCallback(self, actionText): ...
    def _LocationEditorFormWidget__addParameterExpressions(self, paths): ...
    def _LocationEditorFormWidget__addPathEntry(self, element, value): ...
    def _LocationEditorFormWidget__appendNodegraphSelection(self): ...
    def _LocationEditorFormWidget__appendNodegraphSelectionReferences(self, nodes: Incomplete | None = ...): ...
    def _LocationEditorFormWidget__appendScenegraphSelection(self): ...
    def _LocationEditorFormWidget__clearAll(self): ...
    def _LocationEditorFormWidget__clearPanels(self, clearParameters: bool = ...): ...
    def _LocationEditorFormWidget__createExpressionPanel(self, policyNumber): ...
    def _LocationEditorFormWidget__createPathsPanel(self, entry: Incomplete | None = ..., index: int = ...): ...
    def _LocationEditorFormWidget__getPathsPanel(self, createIfDoesNotExist: bool = ...): ...
    def _LocationEditorFormWidget__getValidDroppedNodes(self, mimeData): ...
    def _LocationEditorFormWidget__removePathsPanelIfExists(self): ...
    def _LocationEditorFormWidget__replaceNodegraphSelection(self): ...
    def _LocationEditorFormWidget__replaceNodegraphSelectionReferences(self): ...
    def _LocationEditorFormWidget__replaceWithScenegraphSelection(self): ...
    def _LocationEditorFormWidget__updateExpressionPanelIndices(self): ...
    def _LocationEditorFormWidget__updateGroupTitle(self): ...
    def _participatesInLabelAlignment(self): ...
    def addButtonCheckDragEvent(self, event): ...
    def addButtonDropEvent(self, event): ...
    def buildAddMenu(self, menu): ...
    def event(self, event: PyQt5.QtCore.QEvent) -> bool: ...
    def getScenegraphLocationContext(self): ...
    def getScenegraphSelection(self): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def panelValueChanged(self, index): ...
    def updatePanels(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaFactory as KatanaFactory
import NodegraphAPI
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtCore
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from UI4.FormMaster.BaseParameterPolicy import BaseParameterPolicy
from UI4.KatanaPrefs.KatanaPrefsObject import Prefs as Prefs
from typing import ClassVar, Set, Tuple

NodeEditorCache: NodeEditorCacheClass

class NodeEditorCacheClass(PyQt5.QtCore.QObject):
    _NodeEditorCacheClass__cache: ClassVar[dict] = ...
    def __init__(self, factory: KatanaFactory.KatanaWidgetFactoryClass) -> None: ...
    def _NodeEditorCacheClass__clearAllEditorsForNode(self, nodeHash: int): ...
    def _NodeEditorCacheClass__collapsedEventHandler(self, args): ...
    def _NodeEditorCacheClass__deleteEditor(self, editor: BaseParameterPolicy): ...
    def _NodeEditorCacheClass__on_qApp_aboutToQuit(self): ...
    def buildEditor(self, node: NodegraphAPI.Node) -> BaseParameterPolicy: ...
    def flush(self): ...
    def getEditor(self, node: NodegraphAPI.Node) -> BaseParameterPolicy: ...
    def releaseEditor(self, editor: BaseParameterPolicy): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolib as FnGeolib
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyUtilModule.WorkingSet
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import Nodes3DAPI.ScenegraphManager as ScenegraphManager
import Utils as Utils
from PyUtilModule.WorkingSet import WorkingSet as WorkingSet
from PyUtilModule.WorkingSet.WorkingSet import State
from PyUtilModule.WorkingSetManager import WorkingSetManager as WorkingSetManager
from UI4.Widgets.ModalProcessInterruptWidget import ModalProcessInterruptWidget as ModalProcessInterruptWidget
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class WorkingSetClearStateAction(WorkingSetResetStateAction):
    def __init__(self, text, workingSetName, locationPaths: Incomplete | None = ..., clearGiven: bool = ..., parent: Incomplete | None = ...) -> None: ...

class WorkingSetContextMenu(PyQt5.QtWidgets.QMenu):
    _WorkingSetContextMenu__StateDisplayNames: ClassVar[dict] = ...
    _WorkingSetContextMenu__StateModifierStrings: ClassVar[dict] = ...
    _WorkingSetContextMenu__StatesAndKeyboardModifiers: ClassVar[tuple] = ...
    _WorkingSetContextMenu__StatesToModifierKeys: ClassVar[dict] = ...
    hasCustomActionPropertiesPropertyName: ClassVar[str] = ...
    def __init__(self, workingSetName: str, locationPathOrPaths: str | list[str], showMouseShortcuts: bool = ..., parent: Incomplete | None = ...) -> None: ...
    @classmethod
    def GetModifierKeyStringForState(cls, state: State) -> str | None: ...
    @classmethod
    def GetStateDisplayName(cls, text: str) -> str: ...
    @classmethod
    def GetStateForEventModifiers(cls, eventModifiers): ...
    @classmethod
    def GetStateForKeyboardModifiers(cls, keyboardModifiers: PyQt5.QtCore.Qt.KeyboardModifiers) -> PyUtilModule.WorkingSet.WorkingSet.State | None: ...

class WorkingSetExpandBranchAction(PyQt5.QtWidgets.QAction):
    def __init__(self, text: str, workingSetName: str, locationPaths: Incomplete | None = ..., parent: Incomplete | None = ...) -> None: ...
    def _WorkingSetExpandBranchAction__on_triggered(self): ...

class WorkingSetIconManager:
    StateByInheritanceSuffix: ClassVar[str] = ...
    StateChildExcludedSuffix: ClassVar[str] = ...
    StateChildIncludedSuffix: ClassVar[str] = ...
    StateDisabledSuffix: ClassVar[str] = ...
    StateEmptyPrefix: ClassVar[str] = ...
    StateExcludedPrefix: ClassVar[str] = ...
    StateIncludedPrefix: ClassVar[str] = ...
    StateLockedSuffix: ClassVar[str] = ...
    StateWithChildrenSuffix: ClassVar[str] = ...
    _WorkingSetIconManager__IconBase: ClassVar[str] = ...
    _WorkingSetIconManager__IconByInheritance: ClassVar[str] = ...
    _WorkingSetIconManager__IconCheckmark: ClassVar[str] = ...
    _WorkingSetIconManager__IconChildExcluded: ClassVar[str] = ...
    _WorkingSetIconManager__IconChildIncluded: ClassVar[str] = ...
    _WorkingSetIconManager__IconCross: ClassVar[str] = ...
    _WorkingSetIconManager__IconDisabled: ClassVar[str] = ...
    _WorkingSetIconManager__IconEmpty: ClassVar[str] = ...
    _WorkingSetIconManager__IconExcluded: ClassVar[str] = ...
    _WorkingSetIconManager__IconIncluded: ClassVar[str] = ...
    _WorkingSetIconManager__IconLocked: ClassVar[str] = ...
    _WorkingSetIconManager__IconPrefix: ClassVar[str] = ...
    _WorkingSetIconManager__IconWithChildren: ClassVar[str] = ...
    _WorkingSetIconManager__StateIcons: ClassVar[dict] = ...
    @classmethod
    def GetStateIcon(cls, stateName: str, resolution: Incomplete | None = ...) -> PyQt5.QtGui.QIcon | None: ...
    @classmethod
    def _WorkingSetIconManager__CreateCompositeStateIcon(cls, stateIconComponentNames: list[str], resolution: float | None) -> PyQt5.QtGui.QIcon | None: ...

class WorkingSetIncludeProxyChildren(PyQt5.QtWidgets.QDialog):
    def __init__(self, locationPaths: list, workingSetName: str) -> None: ...
    def _WorkingSetIncludeProxyChildren__cleanup(self): ...
    def _WorkingSetIncludeProxyChildren__idle(self): ...
    def _WorkingSetIncludeProxyChildren__includeLocationsToWorkingSet(self): ...

class WorkingSetResetStateAction(PyQt5.QtWidgets.QAction):
    def __init__(self, text: str, workingSetName: str, locationPaths: Incomplete | None = ..., resetBelow: bool = ..., parent: Incomplete | None = ...) -> None: ...
    def _WorkingSetResetStateAction__on_triggered(self): ...

class WorkingSetStateAction(PyQt5.QtWidgets.QAction):
    def __init__(self, state: State, locationPaths: list[str], workingSetName: str, icon: PyQt5.QtGui.QIcon, text: str, parent: QWidget) -> None: ...
    @classmethod
    def _WorkingSetStateAction__getActionObjectName(cls, state: State) -> str: ...
    def _WorkingSetStateAction__on_triggered(self): ...

class WorkingSetStateCheckableAction(WorkingSetStateAction):
    hasMouseShortcutPropertyName: ClassVar[str] = ...
    isPartiallyCheckedPropertyName: ClassVar[str] = ...
    def __init__(self, state: State, locationPaths: list[str], workingSetName: str, showMouseShortcut: bool, parent: QWidget) -> None: ...
    @classmethod
    def _WorkingSetStateCheckableAction__getActionTextAndIcon(cls, state: State, showMouseShortcut: bool) -> tuple[str, PyQt5.QtGui.QIcon] | Tuple[str, None]: ...
    def isPartiallyChecked(self) -> bool: ...
    def isPartiallyEnabled(self): ...
    def setPartiallyChecked(self, isPartiallyChecked: bool): ...
    def setPartiallyEnabled(self, isPartiallyEnabled): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Widgets.SceneGraphView.DataRoles as DataRoles
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import QItemSelectionModel
import QT4Widgets as QT4Widgets
import QT4Widgets.SortableTreeWidget
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SceneGraphView.SceneGraphLocationTranslation as SceneGraphLocationTranslation
import SceneGraphViewColumn
import UI4.Util.ScenegraphIconManager as ScenegraphIconManager
import Nodes3DAPI.ScenegraphManager as ScenegraphManager
import UI4.Widgets.SceneGraphView.Bridge
import UI4.Widgets.SceneGraphView.ItemDelegates
import Utils as Utils
import typing
from PyUtilModule.WorkingSet import WorkingSet as WorkingSet
from PyUtilModule.WorkingSetManager import WorkingSetManager as WorkingSetManager
from QT4Widgets.SortableTreeWidget import SortableTreeWidget
from UI4.Util.WorkingSets import IncludeProxyChildrenInWorkingSet as IncludeProxyChildrenInWorkingSet
from UI4.Widgets.SceneGraphView.AttributeValuePopupWidgetManager import AttributeValuePopupWidgetManager as AttributeValuePopupWidgetManager
from UI4.Widgets.SceneGraphView.ContextMenuEvent import ContextMenuEvent as ContextMenuEvent
from UI4.Widgets.SceneGraphView.HorizontalHeaderView import HorizontalHeaderView as HorizontalHeaderView
from UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate import ParameterItemDelegate as ParameterItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.WorkingSetItemDelegate import WorkingSetItemDelegate as WorkingSetItemDelegate
from UI4.Widgets.SceneGraphView.SceneGraphHandle import SceneGraphHandle as SceneGraphHandle
from UI4.Widgets.SceneGraphView.SceneGraphLocationTranslation import IsLocationUnderTopLevelLocation as IsLocationUnderTopLevelLocation
from UI4.Widgets.SceneGraphView.SceneGraphViewColumn import SceneGraphColumnTitle as SceneGraphColumnTitle
from UI4.Widgets.SceneGraphView.TreeWidgetItems import LocationTreeWidgetItem as LocationTreeWidgetItem
from UI4.Widgets.SceneGraphView.ViewLink import ViewLink as ViewLink
from UI4.Widgets.WorkingSetWidgets import WorkingSetExpandBranchAction as WorkingSetExpandBranchAction
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ExpandedLocationsRegistry:
    def __init__(self) -> None: ...
    def clear(self, clearCurrent: bool = ...): ...
    def getCurrentlyExpandedLocations(self) -> set[tuple]: ...
    def renameLocation(self, oldLocationPath: str, newLocationPath: str): ...
    def resetCurrentlyExpandedLocations(self): ...
    def setLocationCollapsed(self, locationPath: str, topLevelLocationPath): ...
    def setLocationExpanded(self, locationPath: str, topLevelLocationPath): ...
    def wasLocationExpanded(self, locationPath: str, topLevelLocationPath: str) -> bool: ...
    def __len__(self) -> int: ...

class TreeWidget(SortableTreeWidget):
    PaddingRight: ClassVar[int] = ...
    hideSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, contextMenuEventHandler: typing.Callable, keyPressEventHandler: typing.Callable, dragMoveEventHandler: typing.Callable, dropEventHandler: typing.Callable, aboutToDragEventHandler: typing.Callable, fontChangeEventHandler: typing.Callable, parent: Incomplete | None = ...) -> None: ...
    def _TreeWidget__on_aboutToDrag(self, draggedItems: list[LocationTreeWidgetItem], drag: PyQt5.QtGui.QDrag): ...
    def _TreeWidget__on_customContextMenuRequested(self, pos: PyQt5.QtGui.QPoint): ...
    def _TreeWidget__on_dragMoveEvent(self, event: PyQt5.QtGui.QDragMoveEvent, parentItem: LocationTreeWidgetItem, childItemIndex: int, callbackRecord: QT4Widgets.SortableTreeWidget.CallbackRecord): ...
    def _TreeWidget__on_dropEvent(self, event: PyQt5.QtGui.QDragMoveEvent, parentItem: LocationTreeWidgetItem, childItemIndex: int): ...
    def _TreeWidget__updateWidgetFromFont(self): ...
    def _getCorrespondingPointInNameColumn(self, point: PyQt5.QtCore.QPoint) -> PyQt5.QtCore.QPoint: ...
    def event(self, event: PyQt5.QtCore.QEvent) -> bool: ...
    def fontChange(self, oldFont: PyQt5.QtGui.QFont): ...
    def hideEvent(self, event: PyQt5.QtGui.QHideEvent): ...
    def indexAt(self, point: PyQt5.QtCore.QPoint) -> PyQt5.QtCore.QModelIndex: ...
    def itemDelegateForColumn(self, column): ...
    def keyPressEvent(self, keyEvent: PyQt5.QtGui.QKeyEvent): ...
    def mousePressEvent(self, mouseEvent: PyQt5.QtGui.QMouseEvent): ...
    def resizeColumnToContents(self, column: int): ...
    def selectionCommand(self, index: PyQt5.QtCore.QModelIndex, event: Incomplete | None = ...) -> QItemSelectionModel.SelectionFlags: ...
    def setItemDelegateForColumn(self, column, delegate): ...
    def showEvent(self, event: PyQt5.QtGui.QShowEvent): ...
    def visualRect(self, index: PyQt5.QtCore.QModelIndex) -> PyQt5.QtCore.QRect: ...

class TreeWidgetException(Exception):
    def __init__(self, errorMessage: str) -> None: ...

class TreeWidgetViewLink(ViewLink):
    _TreeWidgetViewLink__erroneousItemDelegateClasses: ClassVar[list] = ...
    _TreeWidgetViewLink__instanceRefs: ClassVar[list] = ...
    _TreeWidgetViewLink__stopAtLocationTypes: ClassVar[tuple] = ...
    _TreeWidgetViewLink__stopCollapsingActionIDs: ClassVar[dict] = ...
    _TreeWidgetViewLink__stopExpandingActionIDs: ClassVar[dict] = ...
    _abc_impl: ClassVar[_abc_data] = ...
    contextName: ClassVar[str] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    def __init__(self, bridge: Bridge[UI4.Widgets.SceneGraphView.Bridge.Bridge], parent: Incomplete | None = ...) -> None: ...
    @classmethod
    def GetStopAtLocationTypes(cls) -> tuple[str, ...]: ...
    @classmethod
    def SetStopAtLocationTypes(cls, *locationTypes): ...
    @classmethod
    def _TreeWidgetViewLink__GetCapitalizedWords(cls, text: str, joiner: str = ...) -> str: ...
    def _TreeWidgetViewLink__aboutToDragEventHandler(self, draggedItems, drag): ...
    def _TreeWidgetViewLink__addChildLocationsToSelectionActionCallback(self): ...
    def _TreeWidgetViewLink__addParentLocationsToSelectionActionCallback(self): ...
    def _TreeWidgetViewLink__buildParameterItemDelegatesDictionary(self, handle, locationPath): ...
    def _TreeWidgetViewLink__collapseBranchesActionCallback(self): ...
    def _TreeWidgetViewLink__collapseBranchesToMultipleActionCallback(self): ...
    def _TreeWidgetViewLink__collapseLocationsActionCallback(self): ...
    def _TreeWidgetViewLink__contextMenuEventHandler(self, event): ...
    def _TreeWidgetViewLink__deleteLocationItem(self, locationItem, subTree): ...
    def _TreeWidgetViewLink__dragMoveEventHandler(self, dragMoveEvent, draggedItems, parentItem, childItemIndex, callbackRecord): ...
    def _TreeWidgetViewLink__dropEventHandler(self, dropEvent, droppedItems, parentItem, childItemIndex): ...
    def _TreeWidgetViewLink__expandBranchesActionCallback(self): ...
    def _TreeWidgetViewLink__expandBranchesToMultipleActionCallback(self): ...
    def _TreeWidgetViewLink__expandLocationsActionCallback(self): ...
    def _TreeWidgetViewLink__expandToAndSelectProxyChildrenCallback(self): ...
    def _TreeWidgetViewLink__fontChangeEventHandler(self): ...
    def _TreeWidgetViewLink__getAncestorLocationsOfType(self, locations, types: set[str]): ...
    def _TreeWidgetViewLink__getHandlesForSelectedItems(self, returnMinimalSet: bool = ...) -> list[None]: ...
    def _TreeWidgetViewLink__getHandlesForTopLevelItems(self) -> list[None]: ...
    def _TreeWidgetViewLink__getParentLocationItem(self, locationPath, subTree): ...
    def _TreeWidgetViewLink__getSubTreeByPath(self, topLevelLocationPath: str) -> dict: ...
    def _TreeWidgetViewLink__isItemExpandedTo(self, treeWidgetItem: PyQt5.QtWidgets.QTreeWidgetItem) -> bool: ...
    def _TreeWidgetViewLink__keyPressEventHandler(self, event): ...
    def _TreeWidgetViewLink__moveSelectionDownActionCallback(self, replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__moveSelectionUpActionCallback(self, replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__onNewScene(self, **kwargs): ...
    def _TreeWidgetViewLink__onSceneLoad(self, **kwargs): ...
    def _TreeWidgetViewLink__on_cacheManager_flush(self): ...
    def _TreeWidgetViewLink__on_event_idle_updateItemDelegates(self, eventType, eventID): ...
    def _TreeWidgetViewLink__on_sceneGraph_locationRenamed(self, eventType: str | None, eventID: object, oldLocationPath: str, newLocationPath: str, selectLocation: bool = ..., replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__on_treeWidgetHeader_sectionMoved(self, logicalIndex, oldVisualIndex, newVisualIndex): ...
    def _TreeWidgetViewLink__on_treeWidgetHeader_sectionPressed(self, logicalIndex): ...
    def _TreeWidgetViewLink__on_treeWidgetHeader_sectionResized(self, logicalIndex, oldSize, newSize): ...
    def _TreeWidgetViewLink__on_treeWidget_destroyed(self): ...
    def _TreeWidgetViewLink__on_treeWidget_hide(self): ...
    def _TreeWidgetViewLink__on_treeWidget_itemCollapsed(self, item): ...
    def _TreeWidgetViewLink__on_treeWidget_itemDoubleClicked(self, item, columnIndex): ...
    def _TreeWidgetViewLink__on_treeWidget_itemExpanded(self, item): ...
    def _TreeWidgetViewLink__on_treeWidget_itemSelectionChanged(self): ...
    def _TreeWidgetViewLink__on_treeWidget_show(self): ...
    def _TreeWidgetViewLink__recursiveShowItems(self, locationItem: LocationTreeWidgetItem): ...
    def _TreeWidgetViewLink__requestOverrideParameters(self, overrideParameters: dict): ...
    def _TreeWidgetViewLink__selectChildLocationsActionCallback(self, replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__selectLocationsByCallback(self, callback, replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__selectParentLocationsActionCallback(self, replaceSelection: bool = ...): ...
    def _TreeWidgetViewLink__showLocationItem(self, locationItem: LocationTreeWidgetItem, show: bool = ...): ...
    def _TreeWidgetViewLink__sortItemChildren(self, treeItem: LocationItem): ...
    def _TreeWidgetViewLink__updateItemChildList(self, treeItem, newItem: bool = ...): ...
    def _TreeWidgetViewLink__updateItemDelegatesForLocations(self, locationItems: list[LocationTreeWidgetItem]): ...
    def _TreeWidgetViewLink__updateLocationItems(self, locationItems: list[LocationTreeWidgetItem]): ...
    def _TreeWidgetViewLink__updateParameterItemDelegateOverrideParameters(self, parameterItemDelegates: None): ...
    def _TreeWidgetViewLink__updateTreeWidgetItem(self, treeWidgetItem, handle): ...
    def addOrUpdateLocation(self, handle, topLevelHandle, potentialChildrenChanged: bool = ...): ...
    def addOrUpdateTopLevelLocation(self, topLevelHandle, potentialChildrenChanged: bool = ...): ...
    def allowMultipleSelection(self) -> bool: ...
    def clearExpandedLocationsRegistry(self, clearCurrent: bool = ...): ...
    def collapseLocation(self, handle, topLevelHandle): ...
    def evaluateFilterRules(self, handle: Incomplete | None = ..., topLevelHandle: Incomplete | None = ...): ...
    def expandLocation(self, handle, topLevelHandle): ...
    def frozenWhenHidden(self) -> bool: ...
    def getAllLocations(self, visibleOnly: bool = ...) -> list[tuple[str, ...]]: ...
    def getChildLocations(self, locationPath, topLevelLocationPath, visibleOnly: bool = ..., allDescendants: bool = ...): ...
    def getItemDelegateTerminalOps(self): ...
    def getOverrideNodeAndParameterName(self, locationPath: str, attributeName: str) -> None: ...
    def getSelectedItems(self): ...
    def getSelectedLocations(self, sortBySceneGraphOrder: bool = ...): ...
    def getUpdateSuppressor(self): ...
    def getWidget(self): ...
    def hideAttributeValuePopupWidget(self, force: bool = ...): ...
    def isLocationExpanded(self, handle, topLevelHandle): ...
    @classmethod
    def registerKeyboardShortcuts(cls): ...
    @classmethod
    def registerKeyboardShortcutsForStopAtLocationTypes(cls): ...
    def removeLocation(self, handle, topLevelHandle, removeFromParent: bool = ...): ...
    def removeTopLevelLocation(self, topLevelHandle): ...
    def resetModel(self): ...
    def saveExpandedLocations(self): ...
    def scrollToLocation(self, handle, topLevelHandle): ...
    def selectChildLocations(self, replaceSelection: bool = ...): ...
    def selectLocations(self, locations, replaceSelection: bool = ...): ...
    def selectParentLocations(self, replaceSelection: bool = ...): ...
    def setAboutToDragCallback(self, callback): ...
    def setAllowMultipleSelection(self, allowMultipleSelection: bool): ...
    def setAttributeDataChildren(self, handle, topLevelHandle, attributeDataChildren): ...
    def setColumnTitles(self, columnTitles: list[SceneGraphColumnTitle[SceneGraphViewColumn.SceneGraphColumnTitle]]): ...
    def setColumnWidths(self, columnWidths): ...
    def setContextMenuEventCallback(self, callback): ...
    def setDragMoveEventCallback(self, callback): ...
    def setDropEventCallback(self, callback): ...
    def setExpandsOnDoubleClick(self, expandsOnDoubleClick): ...
    def setFrozenWhenHidden(self, frozenWhenHidden: bool = ...): ...
    def setKeyPressEventCallback(self, callback): ...
    def setOverrideParameterRequestCallback(self, callback): ...
    def setSelectionChangedCallback(self, callback): ...
    def setSelectionState(self, topLevelLocationPath, locationPath, selected): ...
    def showAttributeValuePopupWidget(self, index: PyQt5.QtCore.QModelIndex, itemDelegate: UI4.Widgets.SceneGraphView.ItemDelegates.ParameterItemDelegate, keepVisible: bool = ...): ...
    def sortChildrenUnderLocation(self, handle, topLevelHandle): ...
    def sortTopLevelLocations(self): ...
    def updateItemDelegates(self): ...
    def updateLocationIcons(self, handle, topLevelHandle): ...
    def updateSelection(self, selectedLocations, deselectedLocations): ...
    def updateViewport(self): ...
    def updateWidget(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import ClassVar, Set, Tuple

class InitialState:
    DEFAULT: ClassVar[float] = ...
    OFF: ClassVar[float] = ...
    ON: ClassVar[float] = ...
    USE_EXISTING_VALUE: ClassVar[float] = ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
from QT4FormWidgets.FWidget import FBoxLayout as FBoxLayout, FButton as FButton, FDisclosureTriangle as FDisclosureTriangle, FLabel as FLabel, FLockIcon as FLockIcon, FMenu as FMenu, FPixmap as FPixmap, FSpacer as FSpacer, FStateBadge as FStateBadge, FSvgIcon as FSvgIcon, FToggleStateBadge as FToggleStateBadge, FWidget as FWidget
from QT4FormWidgets.FormWidget import AlignChildLabelWidths as AlignChildLabelWidths, AlignLeftControlWidths as AlignLeftControlWidths, FormWidget as FormWidget, ScrubbingStates as ScrubbingStates
from QT4FormWidgets.ValuePolicy import ValuePolicyEvent as ValuePolicyEvent
from typing import Set, Tuple

class MenuGroupFormWidget(FormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _MenuGroupFormWidget__doTriangleClicked(self): ...
    def _MenuGroupFormWidget__updateTriangleColor(self): ...
    def _buildControlWidget(self, layout): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _createChildWidget(self, policy, parentWidget, factory, index): ...
    def _firstChildFormWidgetIndex(self): ...
    def _freeze(self): ...
    def _lockChanged(self, state): ...
    def _participatesInLabelAlignment(self): ...
    def _thaw(self): ...
    def buildMenu(self, menu): ...
    def valueChangedEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from UI4.Tabs.UVPanel.Layers.HotkeyLayer import HotkeyLayer as HotkeyLayer
from UI4.Tabs.UVPanel.Layers.SelectionLayer import SelectionLayer as SelectionLayer
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import QT4Color.Swatches as Swatches
from QT4Color.DropType import BuildDragDataFromInfo as BuildDragDataFromInfo, BuildDragPixmapFromColor as BuildDragPixmapFromColor, GetColorDragType as GetColorDragType, GetColorInfoFromDragObject as GetColorInfoFromDragObject, GetColorPolicyDragDict as GetColorPolicyDragDict
from typing import ClassVar, Set, Tuple

class ColorDropWidget(PyQt5.QtWidgets.QWidget):
    clickedSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    dropEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, enableFilmlook: bool = ..., enableNoFilmlookColorSpace: bool = ..., enableMinimalMode: bool = ..., hasAlpha: bool = ..., autoMaxHeight: bool = ..., fstopOffset: float = ...) -> None: ...
    def _ColorDropWidget__getSpotRect(self): ...
    def _ColorDropWidget__startDrag(self): ...
    def customEvent(self, event): ...
    def dragEnterEvent(self, ev): ...
    def dropEvent(self, ev): ...
    def getColor(self): ...
    def getFStopOffset(self): ...
    def mouseMoveEvent(self, ev): ...
    def mousePressEvent(self, ev): ...
    def mouseReleaseEvent(self, ev): ...
    def paintEvent(self, ev): ...
    def setDisplayColor(self, color): ...
    def setDragData(self, data): ...
    def setFilmlookEnabled(self, enabled): ...
    def setMinimalModeEnabled(self, enabled): ...
    def setNoFilmlookColorSpaceEnabled(self, enabled): ...
    def sizeHint(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyOpenColorIO as OCIO
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import ResourceFiles as ResourceFiles
import Utils.WeakMethod as WeakMethod
from ResourceFiles.IconManager import ResourceManager as ResourceManager
from typing import Set, Tuple

__warningregistry__: dict



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
from . import CatalogGLHelpers as CatalogGLHelpers, CommentWidgets as CommentWidgets, DisplayDetailsWidgets as DisplayDetailsWidgets, Layers as Layers, Manipulators as Manipulators, MenuUtils as MenuUtils, MonitorWidget as MonitorWidget
from UI4.Tabs.Monitor.MonitorPanel import MonitorPanel as MonitorPanel
from typing import Set, Tuple

PluginRegistry: list



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import LightingToolsUtils as LightingToolsUtils
from UI4.Widgets.LightingToolsLayer.LightingToolsUtils import GafferThreeListWatcher as GafferThreeListWatcher
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodeGraphView as NodeGraphView
import NodegraphAPI as NodegraphAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import PyQt5.QtGui as QtGui
import UI4 as UI4
import Utils as Utils
from UI4.KatanaPrefs.KatanaPrefsObject import KatanaPrefs as KatanaPrefs
from typing import Set, Tuple

NODEGRAPH_DEFAULT_VIEWSTATE: str
NODEGRAPH_DIMNONCONTRIBUTINGNODES: str
NODEGRAPH_DIMUNCONNECTEDTOVIEWNODE: str
NODEGRAPH_GRIDSNAP: str
NODEGRAPH_NME_DIMDISCONNECTEDNODES: str
NODEGRAPH_NME_DIMEDITEDNODES: str
NODEGRAPH_NME_DIMNEWNODES: str
NODEGRAPH_NME_DIMUNCHANGEDNODES: str
NODEGRAPH_SHOWPAGESCONNECTEDONLY: str
NODEGRAPH_USE_COLOR_FROM_INPUT_PORT_FOR_CONNECTIONS: str
__NODE_TYPES_PARAM_CHANGE_UPDATES_DIM: tuple

class NodeGraphViewStateLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _NodeGraphViewStateLayer__node_cleanUpShapeAttrs_cb(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def _NodeGraphViewStateLayer__node_create_cb(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def _NodeGraphViewStateLayer__node_delete_cb(self, eventType, eventID, node: NodegraphAPI.Node, *args, **kwargs): ...
    def _NodeGraphViewStateLayer__node_setParent_cb(self, eventType, eventID, node: NodegraphAPI.Node, *args, **kwargs): ...
    def _NodeGraphViewStateLayer__node_setShapeAttributes_cb(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def _NodeGraphViewStateLayer__node_setState_cb(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def _NodeGraphViewStateLayer__node_setViewed_cb(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def _NodeGraphViewStateLayer__node_shapeAttrsChanged_cb(self, eventType, eventID, nodeName, oldAttrs, newAttrs, **kwargs): ...
    def _NodeGraphViewStateLayer__onNetworkMaterialEditUpdated(self, eventData): ...
    def _NodeGraphViewStateLayer__on_event_idle(self, *args, **kwargs): ...
    def _NodeGraphViewStateLayer__on_node_setFloating(self, _eventType, _eventID, node: NodegraphAPI.Node): ...
    def _NodeGraphViewStateLayer__on_nodegraph_setCurrentTimeIntermediate(self, eventType: str | None, eventID: object, currentTime: float): ...
    def _NodeGraphViewStateLayer__parameter_finalizeValue_cb(self, args): ...
    def _NodeGraphViewStateLayer__port_setState_cb(self, eventType, eventID, **kwargs): ...
    def _NodeGraphViewStateLayer__registerDirtyCallbacks(self, enabled): ...
    def _NodeGraphViewStateLayer__updateNodeDimming(self): ...
    def idleUpdate(self): ...
    def setBasePosition(self, pos): ...
    def setEnabled(self, enabled): ...

def _nodeCanChangeGraphstate(node: Node3D) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibProducers as FnGeolibProducers
import PyFnGeolibServices as FnGeolibServices
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import Utils as Utils
import collections
from Nodes3DAPI.Node3D import Node3D as Node3D
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class GenericAssign(Node3D):
    _GenericAssign__AttrTypeClasses: ClassVar[dict] = ...
    _HintTrueValues: ClassVar[list] = ...
    __pychecker__: ClassVar[str] = ...
    def __init__(self) -> None: ...
    @staticmethod
    def EscapeValue(value): ...
    @staticmethod
    def HintTrue(hintName: str, hints: dict, default: bool = ...) -> bool: ...
    def _GenericAssign__applyOverridesToInput(self, inputProducer, port: NodegraphAPI.Port, sourceTime, frameTime, applyLocalSettings: bool = ..., varNameSuffix: Incomplete | None = ...): ...
    @staticmethod
    def _decodeHints(text: str) -> dict: ...
    @staticmethod
    def _encodeHints(hints: collections.OrderedDict | dict) -> str: ...
    def _filterAttrList(self, graphState: NodegraphAPI.GraphState, attrList): ...
    def _getAttrRootForIncomingSceneQuery(self): ...
    def _getIncomingSceneOpAndLocation(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction): ...
    def _getOpChain(self, interface): ...
    def _getStaticAttrHintsForIncomingSceneQuery(self, attrPath): ...
    def _getStaticDefaultAttrForIncomingSceneQuery(self, attrPath): ...
    @staticmethod
    def _processGroup(attrlist, currentpath): ...
    def addParameterHints(self, attrName, inputDict): ...
    def canOverride(self, attrName): ...
    def findOverrideParameter(self, path, attrName, time, index: Incomplete | None = ..., editable: bool = ...): ...
    def getArgsParamRoot(self): ...
    def getFixedCELStatement(self): ...
    def setOverride(self, path, attrName, time, attrData, index: Incomplete | None = ..., *args, **kwargs): ...
    def updateArgsParameterDict(self, argsinfo, changeNode: bool = ...): ...
    def updateArgsParameterFile(self, filename: str): ...

def GetArgsFileDict(filename: str) -> collections.OrderedDict: ...
def GetArgsFileXML(filename: str) -> str: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.InputWidgets as InputWidgets
import KatanaResources as KatanaResources
import QT4FormWidgets.PaintingUtils as PaintingUtils
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from QT4FormWidgets.FWidget import FDisclosureTriangle as FDisclosureTriangle
from QT4FormWidgets.FixableBoxLayout import FixableBoxLayout as FixableBoxLayout
from QT4FormWidgets.FormWidget import FormWidget
from QT4Widgets.MenuButton import MenuButton as MenuButton
from UI4.Widgets.IconLabelFrame import IconLabelFrame as IconLabelFrame
from UI4.Widgets.PanelWidget import PanelWidget as PanelWidget
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ParameterExpressionPanel(SortablePanelBase):
    def __init__(self, parent, value: Incomplete | None = ...) -> None: ...
    def _ParameterExpressionPanel__triangleClicked(self, item, state): ...
    def getValue(self): ...
    def setResult(self, result, error: bool = ...): ...
    def setValue(self, value, final: bool = ...): ...
    def updateResultPalette(self, policy): ...

class ParameterSortablePanel(SortablePanelBase):
    def __init__(self, parent, name, policy, factory, panelArgs: Incomplete | None = ...) -> None: ...
    def getWidget(self): ...

class ParameterSortablePanelFormWidget(SortablePanelFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _getPanelForPolicy(self, policy): ...
    def clearParameterChildren(self): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def showPopdown(self, value): ...
    def updatePanels(self): ...

class SortablePanelBase(PanelWidget): ...

class SortablePanelFormWidget(FormWidget):
    _SortablePanelFormWidget__inhibitUpdatePanels: ClassVar[int] = ...
    _SortablePanelFormWidget__needUpdatePanels: ClassVar[bool] = ...
    def __init__(self, parent, policy, factory, showPopdown: bool = ...) -> None: ...
    def _SortablePanelFormWidget__addButtonDragEnterEvent(self, event): ...
    def _SortablePanelFormWidget__addButtonDropEvent(self, event): ...
    def _SortablePanelFormWidget__addMenuCallback(self): ...
    def _SortablePanelFormWidget__dragBoxContextMenuCallback(self, index): ...
    def _SortablePanelFormWidget__dragBoxContextMenuItemCallback(self, index): ...
    def _SortablePanelFormWidget__panelDragInProgress(self, index): ...
    def _SortablePanelFormWidget__panelValueChangedCallback(self, index): ...
    def _SortablePanelFormWidget__renumberPanels(self): ...
    def _allowUpdatePanels(self, allow): ...
    def _buildControlWidget(self, hbox): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _isUpdatePanelsAllowed(self): ...
    def _participatesInLabelAlignment(self): ...
    def _popdownCreated(self, popdown): ...
    def _stealPanel(self, w): ...
    def _thaw(self): ...
    def _updatePanelsLater(self): ...
    def addButtonCheckDragEvent(self, event): ...
    def addButtonDropEvent(self, event): ...
    def addPanel(self, w): ...
    def buildAddMenu(self, menu): ...
    def checkMimeData(self, mimeData): ...
    def deletePanel(self, index, notify: bool = ...): ...
    def destroyPanels(self): ...
    def dragEnterEvent(self, ev, resultList: Incomplete | None = ...): ...
    def dragLeaveEvent(self, ev): ...
    def dragMoveEvent(self, ev, resultList: Incomplete | None = ...): ...
    def dropEvent(self, ev, resultList: Incomplete | None = ...): ...
    def ensurePanelVisible(self, index): ...
    def forceDragFinish(self): ...
    def getAddButton(self): ...
    def getNumPanels(self): ...
    def getPanelFooterLayout(self): ...
    def getPanelHeaderLayout(self): ...
    def getPanelLayout(self): ...
    def getPanels(self): ...
    def getRawDragIndex(self, pos): ...
    def getToolBarCenter(self): ...
    def getToolBarLayout(self): ...
    def getToolBarLeftLayout(self): ...
    def getToolBarRight(self): ...
    def insertPanel(self, w, index): ...
    def movePanel(self, oldPos, newPos): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def panelValueChanged(self, index): ...
    def setAddPanelButtonName(self, name): ...
    def setDragIndex(self, index): ...
    def setDragMinIndex(self, minIndex): ...
    def setLocked(self, state, checkLockOps: bool = ...): ...
    def showPopdown(self, value): ...
    def updatePanels(self): ...
    def valueChangedEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyQt5.QtWidgets
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from PyUtilModule.VirtualKatana import FormMaster as FormMaster
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from UI4.Widgets.PanelScrollArea import PanelScrollArea
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ParameterPanel(BaseTab):
    def __init__(self, parent) -> None: ...
    def _ParameterPanel__findParameterActionCallback(self): ...
    def _ParameterPanel__findPolicyForParameter(self, policy, parameter): ...
    def _ParameterPanel__navigationCallback(self, nodeNames, isBack): ...
    def _ParameterPanel__node_setEdited_callback(self, args: Incomplete | None = ...): ...
    def dragEnterEvent(self, event): ...
    def dropEvent(self, event): ...
    def getPointedWidget(self) -> PyQt5.QtWidgets.QWidget | None: ...
    def parameterEditor_exposeParameter_callback(self, args): ...
    @staticmethod
    def registerKeyboardShortcuts(): ...

class TearoffParameterPanel(PanelScrollArea):
    NonPersistantPanel: ClassVar[bool] = ...
    def __init__(self, parent) -> None: ...
    def _TearoffParameterPanel__node_delete_callback(self, eventName, eventId, **eventArgs): ...
    def getNode(self) -> NodegraphAPI.Node | None: ...
    def setFromWidget(self, widget): ...
    def setNode(self, node: NodegraphAPI.Node): ...

class TearoffParameterTab(BaseTab):
    def __init__(self, parent) -> None: ...
    def getTabTitle(self) -> str | None: ...
    def setFromWidget(self, widget): ...
    def setNode(self, node: NodegraphAPI.Node): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import PyUtilModule.RenderManager as RenderManager
import Utils as Utils
from QT4Widgets.PopdownLabel import PopdownLabel as PopdownLabel
from typing import Set, Tuple

class MyAction(PyQt5.QtWidgets.QAction):
    def __init__(self, mode, *args) -> None: ...
    def _MyAction__activatedCB(self): ...

class ProxyResCombo(PopdownLabel):
    def __init__(self, *args) -> None: ...
    def _ProxyResCombo__sampleRateUpdate_CB(self, *args, **kwargs): ...
    def buildMenu(self, menu: PyQt5.QtWidgets.QMenu): ...
    def updateState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.Editors.PolicyFindPopup as PolicyFindPopup
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
from QT4FormWidgets.GroupFormWidget import GroupFormWidget
from UI4.FormMaster.AttributePolicy import BaseAttributePolicy as BaseAttributePolicy
from UI4.FormMaster.FnAttributePolicy import AttributePolicy as AttributePolicy
from _typeshed import Incomplete
from typing import Set, Tuple

class PeerHeaderGroupFormWidget(GroupFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _buildControlWidget(self, layout): ...
    def _buildLabel(self, labelText, pos: Incomplete | None = ...): ...
    def _buildLock(self, layout): ...
    def _buildStateBadge(self, policy): ...
    def _lockChanged(self, state): ...
    def _participatesInLabelAlignment(self): ...
    def getLabelWidth(self): ...
    def setLabelWidth(self, width): ...
    def showEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
from _typeshed import Incomplete
from typing import Set, Tuple

def CalcCropWindow(rect, dataWindow, renderSettingsCropWindow): ...
def GetLocalNumber(): ...
def GetTmpLocalFile(suffix: str, prefix: str, frame: Incomplete | None = ...) -> str: ...
def SetModeFlags(filename): ...
def ShouldCleanupTmpFiles(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4GLLayerStack.LayerStack as LayerStack
import QT4GLLayerStack.LayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import QT4GLLayerStack.Util as Util
from _typeshed import Incomplete
from typing import Set, Tuple

class FrameAllLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _FrameAllLayer__start(self): ...
    def _FrameAllLayer__update(self, x): ...
    def frameAll(self, no_animation: bool = ..., viewBounds: Incomplete | None = ...): ...
    def isAlreadyFramed(self, viewBounds: Incomplete | None = ...): ...
    def paintGL(self): ...
    def processEvent(self, event): ...
    def setAnimSeconds(self, animSeconds): ...
    def setKeepAspectRatio(self, keepAspectRatio): ...
    def stop(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import Set, Tuple

class ParameterValueOption:
    def __init__(self, *values) -> None: ...
    def __eq__(self, value: str) -> bool: ...
    def __ne__(self, value: str) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyFnGeolib
import Utils as Utils
import abc
import typing
from Nodes3DAPI.PortOpClient import PortOpClient as PortOpClient
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

kCurrentGraphStateId: GraphStateId
kNoGraphStateId: GraphStateId
kViewedNodeId: NodeId
kViewedPortId: PortId

class CallbackPortOpClient(PortOpClient):
    _abc_impl: ClassVar[_abc_data] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    initialGraphStateModifier: Incomplete
    opChangeCallback: Incomplete
    postTraversalGraphStateModifier: Incomplete
    def __init__(self, port: PortId = ..., opChangeCallback: typing.Optional[typing.Callable] = ..., initialGraphStateSpec: GraphStateId = ..., nodeTraversalEndpointsSpec: str = ..., initialGraphStateModifier: Incomplete | None = ..., postTraversalGraphStateModifier: Incomplete | None = ..., applyTerminalOpDelegates: bool = ...) -> None: ...
    @staticmethod
    def _getCallback(weakCallback: typing.Callable): ...
    @staticmethod
    def _getWeakCallback(callback): ...
    def modifyInitialGraphState(self, graphState: NodegraphAPI.GraphState): ...
    def modifyPostTraversalGraphState(self, graphState: NodegraphAPI.GraphState): ...
    def opChanged(self, op, graphState: NodegraphAPI.GraphState, txn): ...

class GraphStateId(str):
    kCurrent: ClassVar[str] = ...
    kNone: ClassVar[str] = ...
    kValues: ClassVar[tuple] = ...
    @classmethod
    def __init__(cls, value: str = ...) -> None: ...

class GraphStateSpec(GraphStateSpec):
    @classmethod
    def __init__(cls, value: GraphStateId = ...) -> None: ...

class NodeId(str):
    kValues: ClassVar[tuple] = ...
    kViewed: ClassVar[str] = ...
    @classmethod
    def __init__(cls, value: str = ...) -> None: ...

class NodeTraversalEndpoints(NodeTraversalEndpointsBase):
    @classmethod
    def __init__(cls, searchPort: NodegraphAPI.Port, startPortOrNode: NodegraphAPI.Port | NodegraphAPI.Node) -> None: ...
    @property
    def searchPort(self): ...
    @property
    def startPortOrNode(self): ...

class NodeTraversalEndpointsSpec(NodeTraversalEndpointsBase):
    @classmethod
    def __init__(cls, searchPortSpec: NodegraphAPI.Port | PortId, startPortOrNodeSpec: NodeId = ...) -> None: ...
    def getNodeTraversalEndpoints(self) -> NodeTraversalEndpoints | None: ...
    @property
    def searchPortSpec(self): ...
    @property
    def startPortOrNodeSpec(self): ...

class PortId(str):
    kValues: ClassVar[tuple] = ...
    kViewed: ClassVar[str] = ...
    @classmethod
    def __init__(cls, value: str = ...) -> None: ...

class PortOpClient(abc.ABC):
    _abc_impl: ClassVar[_abc_data] = ...
    kDefaultInitialGraphStateSpec: ClassVar[GraphStateId] = ...
    kDefaultNodeTraversalEndpointsSpec: ClassVar[str] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    initialGraphStateSpec: Incomplete
    nodeTraversalEndpointsSpec: Incomplete
    port: Incomplete
    def __init__(self, port: PortId = ..., initialGraphStateSpec: GraphStateId = ..., nodeTraversalEndpointsSpec: str = ..., applyTerminalOpDelegates: bool = ...) -> None: ...
    def getNodeTraversalEndpoints(self) -> NodeTraversalEndpoints | None: ...
    def getResolvedPort(self) -> NodegraphAPI.Port | None: ...
    def modifyInitialGraphState(self, graphState: NodegraphAPI.GraphState | None) -> NodegraphAPI.GraphState: ...
    def modifyPostTraversalGraphState(self, graphState: NodegraphAPI.GraphState) -> NodegraphAPI.GraphState: ...
    def opChanged(self, op: PyFnGeolib.GeolibRuntimeOp, graphState: NodegraphAPI.GraphState, txn: PyFnGeolib.GeolibRuntimeTransaction): ...

class PortOpClientRegistry:
    def __init__(self) -> None: ...
    def clientIsRegistered(self, client): ...
    def getClientOp(self, client: PortOpClient) -> PyFnGeolib.GeolibRuntimeOp: ...
    def getClients(self) -> list[PortOpClient]: ...
    def getClientsAndOps(self) -> list[PortOpClient, PyFnGeolib.GeolibRuntimeOp]: ...
    def isClientRegistered(self, client: PortOpClient) -> bool: ...
    def registerClient(self, client: PortOpClient): ...
    def unregisterClient(self, client: PortOpClient): ...
    def updateClientOp(self, client: PortOpClient, op: PyFnGeolib.GeolibRuntimeOp) -> bool: ...

class _NodeTraversalEndpointsBase(tuple):
    _field_defaults: ClassVar[dict] = ...
    _fields: ClassVar[tuple] = ...
    _fields_defaults: ClassVar[dict] = ...
    def __init__(self, _cls, searchPoint, startPoint) -> None: ...
    def _asdict(self): ...
    @classmethod
    def _make(cls, iterable): ...
    def _replace(self, _self, **kwds): ...
    def __getnewargs__(self): ...
    @property
    def searchPoint(self): ...
    @property
    def startPoint(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.PaintingUtils as PaintingUtils
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import ResourceFiles as ResourceFiles
import Utils as Utils
from QT4FormWidgets.FWidget import FWidget as FWidget
from QT4FormWidgets.FixableBoxLayout import FixableBoxLayout as FixableBoxLayout
from ResourceFiles.IconManager import ResourceManager as ResourceManager
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class FBoxLayout(FixableBoxLayout):
    def add(self, w): ...
    def findFWidget(self, point): ...
    def paintFWidgets(self, *args): ...
    def remove(self, w): ...

class FButton(FLabel):
    _FButton__baseBrush: ClassVar[None] = ...
    _FButton__basePen: ClassVar[None] = ...
    def __init__(self, parent, text, font: Incomplete | None = ..., pad: int = ...) -> None: ...
    def _FButton__update(self): ...
    def enterEvent(self, ev): ...
    def leaveEvent(self, ev): ...
    def mousePressEvent(self, event): ...
    def paint(self, painter, width, height): ...
    def setForegroundRole(self, role): ...
    def setLocked(self, isLocked): ...
    def setPreselectionHighlightRole(self, colorRole: Incomplete | None = ...): ...

class FDisclosureTriangle(FWidget):
    _FDisclosureTriangle__duration: ClassVar[float] = ...
    expanded: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, size: int = ..., neverOpen: bool = ..., aspect: float = ...) -> None: ...
    def _FDisclosureTriangle__doTimer(self): ...
    def enterEvent(self, event: PyQt5.QtCore.QEvent): ...
    def fontChange(self, _oldFont: PyQt5.QtGui.QFont): ...
    def isExpanded(self): ...
    def leaveEvent(self, event: PyQt5.QtCore.QEvent): ...
    def mousePressEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    def paint(self, painter, width, height): ...
    def rotation(self): ...
    def setColor(self, color): ...
    def setExpanded(self, state): ...
    def setOutlineColor(self, color): ...
    def setRightPadding(self, rightPadding: int): ...
    def setRotation(self, value): ...
    def setSize(self, size: int): ...
    def sizeHint(self): ...

class FLabel(FWidget):
    def __init__(self, parent, text, font: Incomplete | None = ..., pad: int = ...) -> None: ...
    def getBuddy(self): ...
    def minimumSizeHint(self): ...
    def mousePressEvent(self, event): ...
    def paint(self, painter, width, height): ...
    def setBuddy(self, buddy): ...
    def setColor(self, color): ...
    def setFixedWidth(self, fixedWidth): ...
    def setText(self, text): ...
    def text(self): ...

class FLockIcon(FWidget):
    _FLockIcon__icon: ClassVar[None] = ...
    def __init__(self, parent, size: int = ...) -> None: ...
    def paint(self, painter, width, height): ...
    def setState(self, state): ...
    def sizeHint(self): ...

class FMenu(FWidget):
    menu: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, size: int = ..., text: Incomplete | None = ..., font: Incomplete | None = ..., pad: int = ..., pixmap: Incomplete | None = ...) -> None: ...
    def _FMenu__updateSize(self): ...
    def _displayContextMenu(self, pos: Incomplete | None = ...): ...
    def contextMenuEvent(self, event): ...
    def displayContextMenu(self, pos): ...
    def enterEvent(self, event): ...
    def fillMenu(self, menu): ...
    def invalidate(self): ...
    def leaveEvent(self, event): ...
    def mousePressEvent(self, event): ...
    def paint(self, painter, width, height): ...
    def setFont(self, font): ...
    def setPixmap(self, pixmap): ...
    def setPressed(self, state): ...
    def setText(self, text): ...
    def sizeHint(self): ...

class FPixmap(FWidget):
    def __init__(self, parent, pixmap: Incomplete | None = ...) -> None: ...
    def mousePressEvent(self, event): ...
    def paint(self, painter, width, height): ...
    def pixmap(self): ...
    def setPixmap(self, pixmap): ...
    def sizeHint(self): ...

class FSpacer(FWidget):
    def __init__(self, parent, width) -> None: ...
    def sizeHint(self): ...

class FStateBadge(FWidget):
    def __init__(self, parent, letter, color, size: int = ..., textColor: Incomplete | None = ...) -> None: ...
    def enterEvent(self, event): ...
    def getSize(self): ...
    def leaveEvent(self, event): ...
    def paint(self, painter, width, height, testHover: bool = ...): ...
    def setColors(self, color: Incomplete | None = ..., textColor: Incomplete | None = ...): ...
    def setLetter(self, letter): ...
    def sizeHint(self): ...

class FSvgIcon(FWidget):
    def __init__(self, parent, filename, size: int = ...) -> None: ...
    def paint(self, painter, width, height): ...

class FToggleStateBadge(FWidget):
    def __init__(self, parent, color, size: int = ...) -> None: ...
    def enterEvent(self, event): ...
    def getSize(self): ...
    def leaveEvent(self, event): ...
    def paint(self, painter, width, height, testHover: bool = ...): ...
    def setColor(self, color: Incomplete | None = ...): ...
    def sizeHint(self): ...

class FWidget(PyQt5.QtWidgets.QWidget):
    clicked: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def invalidate(self): ...
    def paint(self, painter, width, height): ...
    def paintEvent(self, event): ...
    def sizeHint(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyUtilModule.RenderManager.RenderGlobals as RenderGlobals
import PyUtilModule.RenderManager as RenderManager
import Utils as Utils
from UI4.Util.IconManager import GetPixmap as GetPixmap
from UI4.Widgets.ToolbarButton import ToolbarButton as ToolbarButton
from typing import Set, Tuple

class LiveRenderViewerCameraButton(ToolbarButton):
    def __init__(self, parent: PyQt5.QtWidgets.QWidget, viewportWidget: ViewportWidget) -> None: ...
    def _LiveRenderViewerCameraButton__on_toggled(self): ...
    def _LiveRenderViewerCameraButton__on_viewer_liveRenderFromViewerCameraChanged(self, eventType: str, eventID: object, followViewportCamera: Bool, viewportWidget: ViewportWidget): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConditionalStateGrammar as ConditionalStateGrammar
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibServices as FnGeolibServices
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import RenderingAPI as RenderingAPI
from ConditionalStateGrammar.Parser import ParseCSG as ParseCSG
from Nodes3DAPI.CreateUtil import BuildChild as BuildChild
from Nodes3DAPI.InputGraphBasedCacheManager import InputGraphBasedCacheManager as InputGraphBasedCacheManager
from Nodes3DAPI.Node3D import Node3D as Node3D
from Nodes3DAPI.ShadingNodeUtil import GetConnectionNames as GetConnectionNames, ReportError as ReportError
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

_ExtraHints: dict
_Parameter_XML: str

class NetworkMaterialNode(Node3D):
    inputsCacheManager: ClassVar[InputGraphBasedCacheManager] = ...
    kNodeGraphViewLayoutVersion: ClassVar[int] = ...
    def __init__(self) -> None: ...
    def _NetworkMaterialNode__buildLayoutAttrsOpArgsIfRequired(self, argsBuilder): ...
    def _NetworkMaterialNode__buildOpArgs(self, graphState: NodegraphAPI.GraphState, inputNames): ...
    def _NetworkMaterialNode__checkStructuredPortConnection(self, sPort, graphState: NodegraphAPI.GraphState, connectionsAttr, nodeDict, interfaceDict, interfaceOrder, visitedNodes): ...
    @staticmethod
    def _NetworkMaterialNode__getAllShadingConnectionNodes(node: NodegraphAPI.Node) -> list[NodegraphAPI.Node]: ...
    def _addArrayConnectionOverrides(self, node: NodegraphAPI.Node, frameTime, nodeDict, visitedNodes): ...
    def _addNodeOverrides(self, node: NodegraphAPI.Node, graphState: NodegraphAPI.GraphState, nodeDict, interfaceDict, interfaceOrder, visitedNodes, error: Incomplete | None = ...): ...
    def _getIncomingSceneOpAndLocation(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction): ...
    def _getOp(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState, transaction): ...
    def addParameterHints(self, attrName, inputDict): ...
    def addShaderInputPort(self, rendererName: str, shaderType: str) -> NodegraphAPI.Port: ...
    def generateShaderInputPortName(self, rendererName: str, shaderType: str) -> str: ...
    def getScenegraphLocation(self, frameTime: Incomplete | None = ...) -> str | None: ...
    def invalidateLayout(self): ...
    def isResetPossible(self): ...
    def validateConnection(self, otherOutPort, myInPort, errorMessages: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolib as FnGeolib
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyFnGeolib
import Utils as Utils
import typing
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AttributeHistory:
    LocationPathHistoryEntryPrefix: ClassVar[str] = ...
    MAX_TIME_PER_IDLE: ClassVar[float] = ...
    OpTypeHistoryEntryPrefix: ClassVar[str] = ...
    ParentDelegateTypes: ClassVar[tuple] = ...
    STOP_AT_ATTR_CREATION: ClassVar[int] = ...
    STOP_AT_ATTR_SET: ClassVar[int] = ...
    STOP_AT_END: ClassVar[int] = ...
    STOP_IMMEDIATELY: ClassVar[int] = ...
    def __init__(self, locationPath: str, attributeName: str, startNode: Incomplete | None = ..., startOp: Incomplete | None = ..., startPort: Incomplete | None = ..., startGraphState: Incomplete | None = ..., delegateNodes: bool = ...) -> None: ...
    @classmethod
    def GetHistoryDelegateNode(cls, node: NodegraphAPI.Node): ...
    @classmethod
    def IsLocationPath(cls, historyEntry): ...
    @classmethod
    def IsOpType(cls, historyEntry): ...
    def _AttributeHistory__appendInputOpsToQueue(self, op: FnGeolibOp): ...
    def _AttributeHistory__appendToHistory(self, op: FnGeolibOp, attributes: dict[str, PyFnAttribute] | None, locationPath: str) -> bool: ...
    def _AttributeHistory__deleteClient(self, clientKey): ...
    def _AttributeHistory__evaluateNextOp(self): ...
    def _AttributeHistory__findLocationPathWithAttribute(self, attrDict: dict, locationPath, includeGlobals) -> str | None: ...
    def _AttributeHistory__getCachedHistory(self) -> list[tuple[str, PyFnAttribute]]: ...
    def _AttributeHistory__getParentLocation(self, locationPath): ...
    def _AttributeHistory__on_event_idle(self, eventType, eventID): ...
    def _AttributeHistory__opToHistoryEntry(self, op: FnGeolibOp) -> str: ...
    def _AttributeHistory__processLocationData(self, op: FnGeolibOp, locationPath: str, locationData: PyFnGeolib.LocationData | None) -> bool: ...
    def _AttributeHistory__processLocationDataAsync(self, op: FnGeolibOp, locationPath: str, locationData: PyFnGeolib.LocationData | None) -> bool: ...
    def _AttributeHistory__setCallbacks(self, completeCallback: typing.Callable, updateCallback: typing.Callable): ...
    def _AttributeHistory__setClientOp(self, clientKey, op): ...
    def _AttributeHistory__shouldStopAtAttribute(self, attribute): ...
    def _AttributeHistory__updateHistory(self) -> bool: ...
    def _AttributeHistory__walkOpTree(self, synchronous: bool = ...): ...
    def clear(self): ...
    def freeze(self): ...
    def getAttributeCreationNode(self, callback: Incomplete | None = ..., updateCallback: typing.Optional[typing.Callable] = ...) -> tuple[Nodes3DAPI.Node3D, PyFnAttribute] | Tuple[None, None]: ...
    def getAttributeInheritance(self, callback: Incomplete | None = ..., updateCallback: typing.Optional[typing.Callable] = ...) -> list[tuple[str, PyFnAttribute] | None]: ...
    def getAttributeName(self): ...
    def getAttributeSetNode(self, callback: Incomplete | None = ..., updateCallback: typing.Optional[typing.Callable] = ...) -> tuple[Nodes3DAPI.Node3D, PyFnAttribute] | None: ...
    def getLocationPath(self): ...
    def getNodeHistory(self, callback: Incomplete | None = ..., updateCallback: typing.Optional[typing.Callable] = ...): ...
    def getStartNode(self): ...
    def isFrozen(self): ...
    def thaw(self): ...

class _AsyncOpManager:
    MAX_UPSTREAM_CLIENTS: ClassVar[int] = ...
    def __init__(self, runtime, evaluatedOpInputs, startOp, locationPath) -> None: ...
    def activateOpInfo(self, opInfo): ...
    def addOpInfo(self, opInfo): ...
    def continueToInputOp(self) -> bool: ...
    def getCurrentOpInfo(self): ...
    def getOpInfo(self, op): ...
    def tryNextInput(self) -> bool: ...
    def updateActiveClientList(self): ...

class _OpInfo:
    def __init__(self, op, opManager) -> None: ...
    def getInputOpInfos(self): ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...

def GetAttributeCreationNode(locationPath: str, attributeName: str, startNode: Incomplete | None = ..., startOp: Incomplete | None = ..., startPort: Incomplete | None = ..., startGraphState: Incomplete | None = ..., delegateNodes: bool = ...) -> tuple[Nodes3DAPI.Node3D, PyFnAttribute] | Tuple[None, None]: ...
def GetAttributeInheritance(locationPath: str, attributeName: str, startNode: Incomplete | None = ..., startOp: Incomplete | None = ..., startPort: Incomplete | None = ..., startGraphState: Incomplete | None = ...) -> list[tuple[str, PyFnAttribute] | None]: ...
def GetAttributeSetNode(locationPath: str, attributeName: str, startNode: Incomplete | None = ..., startOp: Incomplete | None = ..., startPort: Incomplete | None = ..., startGraphState: Incomplete | None = ..., delegateNodes: bool = ...) -> tuple[Nodes3DAPI.Node3D, PyFnAttribute] | Tuple[None, None]: ...
def GetNodeHistory(locationPath: str, attributeName: str, startNode: Incomplete | None = ..., startOp: Incomplete | None = ..., startPort: Incomplete | None = ..., startGraphState: Incomplete | None = ..., delegateNodes: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.ValuePolicy
import QT4FormWidgets.ValuePolicy as ValuePolicy
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class PythonDictValuePolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    RESERVED_NAMES: ClassVar[set] = ...
    def __init__(self, name, dictionary, parent: Incomplete | None = ..., childOrderList: Incomplete | None = ..., childHintDict: Incomplete | None = ...) -> None: ...
    def _PythonValuePolicy__confirmChildHintDict(self, childHints): ...
    def _PythonValuePolicy__confirmOrderList(self, orderList): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getName(self): ...
    def getNumChildren(self): ...
    def getType(self): ...
    def getWidgetHints(self): ...

class PythonGroupPolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    def __init__(self, name, parent: Incomplete | None = ...) -> None: ...
    def addChildPolicy(self, policy): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getName(self): ...
    def getNumChildren(self): ...
    def getStateAppearance(self, state: set): ...
    def getType(self): ...
    def getWidgetHints(self): ...
    def removeChildPolicy(self, policy): ...
    def setWidgetHints(self, hints): ...

class PythonValuePolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    RESERVED_NAMES: ClassVar[set] = ...
    def __init__(self, name, dictionary, parent: Incomplete | None = ..., childOrderList: Incomplete | None = ..., childHintDict: Incomplete | None = ...) -> None: ...
    def _PythonValuePolicy__confirmChildHintDict(self, childHints): ...
    def _PythonValuePolicy__confirmOrderList(self, orderList): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getName(self): ...
    def getNumChildren(self): ...
    def getType(self): ...
    def getWidgetHints(self): ...

class _ArrayPythonPolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    def __init__(self, name, valueList, parent) -> None: ...
    def _ArrayPythonPolicy__buildChildren(self): ...
    def getArrayChild(self, index): ...
    def getArraySize(self): ...
    def getName(self): ...
    def getType(self): ...
    def getWidgetHints(self): ...
    def setArraySize(self, size): ...

class _NumberPythonPolicy(_StringPythonPolicy):
    Types: ClassVar[tuple] = ...

class _StringPythonPolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    Types: ClassVar[tuple] = ...
    def __init__(self, name, dictionary, parent) -> None: ...
    def getName(self): ...
    def getType(self): ...
    def getValue(self): ...
    def getWidgetHints(self): ...
    def setValue(self, value, final: bool = ..., emit: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import KatanaInfo as Katana, KatanaInfo as KatanaInfo, MachineInfo as MachineInfo
from MachineInfo.MachineInfo import GetFreeSpaceB as GetFreeSpaceB, GetNumberOfProcessors as GetNumberOfProcessors, GetTotalMemoryKB as GetTotalMemoryKB, GetUsedSpaceB as GetUsedSpaceB
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyUtilModule.RenderManager.InteractiveRenderDelegateManager as InteractiveRenderDelegateManager
import NodegraphAPI as NodegraphAPI
import PyUtilModule.RenderManager.RenderGlobals as RenderGlobals
import Utils as Utils
from PyUtilModule.RenderManager.Constants import RenderModes as RenderModes
from PyUtilModule.RenderManager.Exceptions import RenderingException as RenderingException
from typing import Set, Tuple

def ApplyInteractiveRenderDelegates(renderPort: NodegraphAPI.Port): ...
def BuildRenderPortsAndGraphStatesList(settings: RenderingSettings, sourcePorts: list[int | str], sourceNodes: NodegraphAPI.Node) -> list[NodegraphAPI.Port]: ...
def CollapseNodesAndPorts(node: NodegraphAPI.Node, port: NodegraphAPI.Port, nodeList, portList): ...
def CreateRenderNodeParameters(node: NodegraphAPI.Node, settings: RenderingSettings, renderComponents: RenderComponents) -> list[NodegraphAPI.Parameter]: ...
def DisableKatanaNodeGraphEventProcessing(): ...
def DisconnectInteractiveRenderDelegates(renderPort: NodegraphAPI.Port, blindData: dict): ...
def GetSourcePortsAndNamesFromLists(nodeList: list[NodegraphAPI.Node | str], portList: list[str], renderMethodType: str) -> list[None]: ...
def InteractiveRenderDelegatesApplied(renderPort: NodegraphAPI.Port) -> NodegraphAPI.Port: ...
def IsScriptRender(renderPortsAndGraphStates: list[NodegraphAPI.Port]) -> bool: ...
def NotifyIfInteractiveRerender(renderAction, nodeList): ...
def _FilterAllNodeGraphEvents(eventType, eventID, *args, **kwargs): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyUtilModule.IRFs as IRFs
import NodegraphAPI as NodegraphAPI
import NodegraphAPI.RenderFilter
from typing import Set, Tuple

class Delegate:
    def accepts(self, node: NodegraphAPI.Node, renderItem): ...
    def begin(self, node: NodegraphAPI.Node, renderItem): ...
    def end(self, node: NodegraphAPI.Node, renderItem, blindData): ...

class IrfException(Exception): ...

class RenderFiltersDelegate(Delegate):
    def accepts(self, node: NodegraphAPI.Node, renderItem): ...
    def begin(self, node: NodegraphAPI.Node, renderItem): ...
    def end(self, node: NodegraphAPI.Node, renderItem, blindData): ...
    def getActiveRenderFilterNodes(self) -> list[NodegraphAPI.RenderFilter.RenderFilterNode]: ...
    def setActiveRenderFilterNodes(self, nodes: list[NodegraphAPI.RenderFilter.RenderFilterNode]): ...

def GetRenderFiltersDelegate(): ...
def RegisterDelegate(nodeType, delegate): ...
def RenderPreprocessBegin(renderItem): ...
def RenderPreprocessEnd(renderItem, blindData): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.Conditional as Conditional
import QT4FormWidgets.FormWidgetTools as FormWidgetTools
import QT4FormWidgets.HintUtils as HintUtils
import QT4FormWidgets.InputWidgets as InputWidgets
import QT4FormWidgets.MenuUtils as MenuUtils
import QT4FormWidgets.OpenState as OpenState
import QT4FormWidgets.PaintingUtils as PaintingUtils
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4FormWidgets.FWidget import FDisclosureTriangle as FDisclosureTriangle
from QT4FormWidgets.FixableBoxLayout import FixableBoxLayout as FixableBoxLayout
from QT4FormWidgets.FormClose import FormClose as FormClose
from QT4FormWidgets.FormWidgetLabel import FormWidgetLabel as FormWidgetLabel
from QT4FormWidgets.MultiStateBadge import MultiStateBadge as MultiStateBadge, ToggleStateBadge as ToggleStateBadge
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class FormWidget(PyQt5.QtWidgets.QWidget):
    HIGHLIGHTSTYLE_ALWAYS: ClassVar[int] = ...
    HIGHLIGHTSTYLE_NONE: ClassVar[int] = ...
    HIGHLIGHTSTYLE_ROLLOVER: ClassVar[int] = ...
    HIGHLIGHTSTYLE_TRANSPARENT: ClassVar[int] = ...
    LABELPOS_LEFT: ClassVar[int] = ...
    LABELPOS_RIGHT: ClassVar[int] = ...
    TEXTCOLOR_ERROR: ClassVar[PyQt5.QtGui.QColor] = ...
    TEXTCOLOR_WARNING: ClassVar[PyQt5.QtGui.QColor] = ...
    _FormWidget__DRAG_THRESHOLD: ClassVar[int] = ...
    _FormWidget__HIGHLIGHTSTYLE_LIMIT: ClassVar[int] = ...
    _FormWidget__allowStickyScrub: ClassVar[bool] = ...
    _FormWidget__backColorBrush: ClassVar[None] = ...
    enterEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    leaveEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showPopdownChange: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    visibleStateChange: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _FormWidget__buildPopdownArea(self): ...
    def _FormWidget__copy(self): ...
    def _FormWidget__doTriangleClicked(self, item, state): ...
    def _FormWidget__finishScrubbing(self, event: Incomplete | None = ..., shouldCommit: bool = ...): ...
    def _FormWidget__focusChanged(self, old, new): ...
    def _FormWidget__freezeConditionalVis(self): ...
    def _FormWidget__getNodeAndParameterFromWidgetHints(self) -> tuple: ...
    def _FormWidget__lockOpStateCallback(self, state): ...
    def _FormWidget__on_destroyed(self): ...
    def _FormWidget__setTriangleState(self, state): ...
    def _FormWidget__startDrag(self): ...
    def _FormWidget__thawConditionalVis(self): ...
    def _FormWidget__updateCloseButtonIfNeeded(self): ...
    def _FormWidget__updateJumpToParameterButtonIfNeeded(self): ...
    def _FormWidget__visOpStateCallback(self, state): ...
    def _buildCloseButton(self, policy): ...
    def _buildControlWidget(self, layout): ...
    def _buildHelpButton(self, policy): ...
    def _buildInfoWidget(self): ...
    def _buildJumpToParameterButton(self) -> PyQt5.QtWidgets.QPushButton | None: ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _buildStateBadge(self, policy): ...
    def _buildTopAreaLayout(self, layout): ...
    def _buildWrench(self, policy): ...
    def _freeze(self): ...
    def _lockChanged(self, state): ...
    def _participatesInLabelAlignment(self): ...
    def _popdownCreated(self, popdown): ...
    def _popupMenuCreated(self, menu): ...
    def _removeFormWidget(self, child): ...
    def _removeWrench(self, wrench): ...
    def _resetScrubbing(self): ...
    def _thaw(self): ...
    def _updateStateBadge(self): ...
    def _updateWithHints(self, originalHints, hints): ...
    def addFormWidgetChild(self, child, atIndex: Incomplete | None = ...): ...
    def alignChildLabelWidths(self): ...
    def alignLeftControlWidths(self): ...
    def checkMimeData(self, mimeData): ...
    def contextMenuEvent(self, event): ...
    def customEvent(self, event): ...
    def dragEnterEvent(self, event): ...
    def dropEvent(self, event): ...
    def enterEvent(self, event): ...
    def event(self, event: PyQt5.QtCore.QEvent) -> bool: ...
    def eventFilter(self, parent, event): ...
    @classmethod
    def getCoPolicyPathDict(cls, policy: QT4FormWidgets.AbstractValuePolicy, alteredPolicyName: Incomplete | None = ...): ...
    def getControlLayout(self): ...
    def getControlWidgets(self): ...
    def getCustomHighlightColor(self): ...
    @classmethod
    def getDefaultBoldLabelFont(cls): ...
    @classmethod
    def getDefaultLabelFont(cls): ...
    @classmethod
    def getDefaultSecondaryLabelFont(cls): ...
    def getFactory(self): ...
    def getFormWidgetChild(self, name): ...
    def getFormWidgetChildren(self): ...
    def getFormWidgetParent(self): ...
    def getHighlightStyle(self): ...
    def getLabel(self) -> FormWidgetLabel: ...
    def getLabelText(self): ...
    def getLabelWidth(self): ...
    def getLeftControlFWidgets(self): ...
    def getLeftControlWidth(self): ...
    def getLeftLabelFWidgets(self): ...
    def getMaximumLeftControlWidgetWidth(self): ...
    def getMimeData(self) -> PyQt5.QtCore.QMimeData: ...
    def getPopdownEnclosureDrawn(self): ...
    def getPopdownIndent(self): ...
    def getPopdownWidget(self): ...
    def getRightControlFWidgets(self): ...
    def getRightLabelFWidgets(self): ...
    def getScrubbingState(self) -> QT4FormWidgets.ScrubbingStates: ...
    def getStateBadge(self): ...
    def getTopologyDifferences(self): ...
    def getTriangle(self): ...
    def getValuePolicy(self): ...
    def getWidgetHints(self): ...
    def hideEvent(self, event): ...
    @classmethod
    def hintTrue(cls, hintName, hints, default: Incomplete | None = ...): ...
    def isFrozen(self): ...
    def isLocked(self): ...
    def isPopdownShown(self): ...
    def isScrubbing(self) -> bool: ...
    def isTriangleShown(self): ...
    def keyPressEvent(self, event): ...
    def keyReleaseEvent(self, event): ...
    def labelSliderBegin(self, pos, modifiers): ...
    def labelSliderDrag(self, pos, modifiers): ...
    def labelSliderEnd(self, pos, modifiers): ...
    def leaveEvent(self, event): ...
    def mouseMoveEvent(self, event): ...
    def mousePressEvent(self, event): ...
    def mouseReleaseEvent(self, event): ...
    def paintEvent(self, event): ...
    def revealFormWidgetChild(self, path: str | list[str]) -> FormWidget: ...
    @classmethod
    def setAllowStickyScrub(cls, allowStickyScrub): ...
    def setHighlightStyle(self, style): ...
    def setInfo(self, text, color: Incomplete | None = ...): ...
    def setLabelText(self, text): ...
    def setLabelWidth(self, width): ...
    def setLeftControlWidth(self, width: int): ...
    def setLocked(self, state, checkLockOps: bool = ...): ...
    def setMimeData(self, mimeData, dropAction): ...
    def setPopdownEnclosureDrawn(self, value): ...
    def setPopdownIndent(self, value): ...
    def setTemporaryBackgroundColor(self, color): ...
    def setVisible(self, state, checkVisOps: bool = ...): ...
    def showEvent(self, event): ...
    def showPopdown(self, value): ...
    def showTriangle(self, flag): ...
    def supportsStickyScrub(self) -> bool: ...
    def topAreaLayoutContains(self, pos): ...
    def updateGeometry(self): ...
    def valueChangedEvent(self, event): ...
    def __lt__(self, other) -> bool: ...

class ScrubbingStates:
    NONE: ClassVar[int] = ...
    NORMAL: ClassVar[int] = ...
    STANDBY: ClassVar[int] = ...
    STICKY: ClassVar[int] = ...

def AlignChildLabelWidths(widgets): ...
def AlignLeftControlWidths(widgets): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import QT4Color.Swatches as Swatches
from QT4Color.ColorPolicy import CreateColorPolicy as CreateColorPolicy, DoesColorPolicyEnableFilmlook as DoesColorPolicyEnableFilmlook, DoesColorPolicyEnableNoFilmlookColorSpace as DoesColorPolicyEnableNoFilmlookColorSpace, DoesColorPolicyHaveAlpha as DoesColorPolicyHaveAlpha, DoesColorPolicyRestrictComponents as DoesColorPolicyRestrictComponents, GetColorPolicyChildren as GetColorPolicyChildren, GetColorPolicyRGBA as GetColorPolicyRGBA, GetDefaultColorComponentTab as GetDefaultColorComponentTab, SetColorPolicyEnableFilmlook as SetColorPolicyEnableFilmlook, SetColorPolicyEnableFilmlookDefault as SetColorPolicyEnableFilmlookDefault, SetColorPolicyEnableNoFilmlookColorSpaceDefault as SetColorPolicyEnableNoFilmlookColorSpaceDefault, SetColorPolicyRGBA as SetColorPolicyRGBA, SetColorPolicyRestrictComponents as SetColorPolicyRestrictComponents, SetColorPolicyRestrictComponentsDefault as SetColorPolicyRestrictComponentsDefault
from typing import Set, Tuple

def BuildDragDataFromInfo(**dragDict): ...
def BuildDragPixmapFromColor(color, enableFilmlook: bool = ..., enableNoFilmlookColorSpace: bool = ..., hasAlpha: bool = ...): ...
def GetColorDragType(): ...
def GetColorInfoFromDragObject(dragObject): ...
def GetColorPolicyDragDict(policy): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyXmlIO as PyXmlIO
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import UI4.Widgets.SortablePanel
from UI4.Widgets.SortablePanel import ParameterExpressionPanel as ParameterExpressionPanel, ParameterSortablePanel as ParameterSortablePanel, ParameterSortablePanelFormWidget as ParameterSortablePanelFormWidget, SortablePanelBase as SortablePanelBase, SortablePanelFormWidget as SortablePanelFormWidget
from typing import Set, Tuple

class SortableArrayEditor(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _SortableArrayEditor__getArrayChild(self, i): ...
    def _SortableArrayEditor__isArray(self): ...
    def buildAddMenu(self, menu): ...
    def getArrayPanelClass(self): ...
    def newEntry(self): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def updatePanels(self): ...

class SortableArrayPanel(UI4.Widgets.SortablePanel.SortablePanelBase):
    def __init__(self, parent, policy, factory) -> None: ...
    def getFormWidget(self): ...
    def getPolicy(self): ...

class _HintWrapperPolicyProxy(QT4FormWidgets.ValuePolicy.ValuePolicyProxy):
    def __init__(self, policy, extraHints) -> None: ...
    def getWidgetHints(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import Set, Tuple

class ViewportPane(PyQt5.QtWidgets.QFrame):
    viewportWidget: Incomplete
    def __init__(self, parent: Incomplete | None = ...) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import UI4.FormMaster as FormMaster
import UI4.Util.IconManager as IconManager
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtGui
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets.SortableTreeWidget
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Util.ScenegraphIconManager as ScenegraphIconManager
import Utils as Utils
import UI4.Widgets as Widgets
from QT4FormWidgets.FormWidget import FormWidget
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AttrSourceStateSource(PyQt5.QtCore.QObject):
    def __init__(self, prefix, attrSource) -> None: ...
    def getHints(self, path): ...

class ShadingNetworkMaterialInterfaceOrderFormWidget(FormWidget):
    class _ShadingNetworkMaterialInterfaceOrderFormWidget__TreeDelegate(QT4Widgets.SortableTreeWidget.SortableTreeWidgetItemDelegate):
        GRAYED_PALETTE: ClassVar[None] = ...
        def _TreeDelegate__buildPalette(self, palette): ...
        def paint(self, painter, option, index): ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__aboutToDragCallback(self, items, dragObject): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__addTreePolicy(self, parent, policy, leafIndex, openNames, interfaceAttr, parentPath: str = ...): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__buildFromCurrentState(self, onlyLocals: bool = ...): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__dragMoveEventCallback(self, event, parent, index, callbackRecord): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__dropEventCallback(self, event, parent, index): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__getDragSlices(self, items): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__getFirstNonPageIndex(self, item): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__getSliceCount(self, item, count: Incomplete | None = ...): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__jumpToSource(self): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__listMousePressEvent(self, event): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__menuAboutToShow(self, menu, selectedItems: Incomplete | None = ...): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__treeContextMenuCallback(self, event: PyQt5.QtGui.QContextMenuEvent): ...
    def _ShadingNetworkMaterialInterfaceOrderFormWidget__updateTree(self): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _buildTopAreaLayout(self, layout): ...
    def showPopdown(self, value: bool): ...
    def valueChangedEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import NodegraphAPI as NodegraphAPI
import Nodes2DAPI as Nodes2DAPI
import PyXmlIO as PyXmlIO
import Utils as Utils
from typing import Set, Tuple

def AddInput(node: NodegraphAPI.Node): ...
def AddPostScript(node: NodegraphAPI.Node): ...
def DeleteInput(node: NodegraphAPI.Node, index): ...
def DeletePostScript(node: NodegraphAPI.Node, index): ...
def EnableFirstInput(node: NodegraphAPI.Node): ...
def GetActiveViewMask(node: NodegraphAPI.Node): ...
def IsParameterTreeOutOfDate(node: NodegraphAPI.Node): ...
def ReorderInput(node: NodegraphAPI.Node, index, newIndex): ...
def ReorderPostScript(node: NodegraphAPI.Node, index, newIndex): ...
def SyncAllOutputPorts(): ...
def SyncOutputPorts(node: NodegraphAPI.Node): ...
def SyncParametersToPorts(node: NodegraphAPI.Node): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import ConfigurationAPI_cmodule as Configuration
import Utils.EventModule as EventModule
import PyFnGeolibServices.ExpressionMath as ExpressionMath
import PyFnAttribute as FnAttribute
import NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import NodegraphAPI.NodegraphGlobals as NodegraphGlobals
import PyFCurve as PyFCurve
import PyResolutionTableFn as ResolutionTable
import Utils as Utils
from PyFnGeolibServices.ExpressionMath import function as function
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ProjectSettings:
    __pychecker__: ClassVar[str] = ...
    def __init__(self) -> None: ...
    def __getattr__(self, attr): ...
    @property
    def assetID(self): ...
    @property
    def dir(self): ...
    @property
    def file(self): ...
    @property
    def originalAssetID(self): ...

class _ParamRef:
    def __init__(self, param) -> None: ...
    def getChildren(self): ...
    def getNode(self) -> NodegraphAPI.Node: ...
    def getNodeName(self): ...
    def getParent(self): ...
    def isNumber(self): ...
    def __abs__(self): ...
    def __add__(self, other): ...
    def __bool__(self) -> bool: ...
    def __call__(self, time: Incomplete | None = ...): ...
    def __div__(self, other): ...
    def __divmod__(self, other): ...
    def __eq__(self, other) -> bool: ...
    def __float__(self) -> float: ...
    def __floordiv__(self, other): ...
    def __getattr__(self, attr): ...
    def __getitem__(self, key): ...
    def __int__(self) -> int: ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __lt__(self, other) -> bool: ...
    def __mod__(self, other): ...
    def __mul__(self, other): ...
    def __neg__(self): ...
    def __pos__(self): ...
    def __pow__(self, other, modulo: Incomplete | None = ...): ...
    def __radd__(self, other): ...
    def __rdiv__(self, other): ...
    def __rdivmod__(self, other): ...
    def __rfloordiv__(self, other): ...
    def __rmod__(self, other): ...
    def __rmul__(self, other): ...
    def __rpow__(self, other): ...
    def __rsub__(self, other): ...
    def __rtruediv__(self, other): ...
    def __sub__(self, other): ...
    def __truediv__(self, other): ...

def FCurveLookup(curveFile, curveName, frame): ...
def GetExpressionGlobalValue(name: str): ...
def SetExpressionGlobalValue(name: str, value): ...
def SetupExpressionGlobalScopeFromRootNode(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import Set, Tuple

def GetFreeSpaceB(dirName): ...
def GetNumberOfProcessors(quiet: bool = ..., raiseExc: bool = ...): ...
def GetTotalMemoryKB(quiet: bool = ..., raiseExc: bool = ...): ...
def GetUsedSpaceB(dirName): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Nodes3DAPI.DynamicParameterUtil as DynamicParameterUtil
import PyFnGeolibServices as FnGeolibServices
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import PyFnAttribute
from Nodes3DAPI.Node3D import Node3D as Node3D
from typing import Set, Tuple

_ExtraHints: dict
_Parameter_XML: str

class LocationCreate(Node3D):
    def __init__(self) -> None: ...
    def _getOpChain(self, interface): ...
    def addAttributeEntry(self, attr: PyFnAttribute.Attribute, path: str, hints: dict): ...
    def addParameterHints(self, attrName, inputDict): ...
    def deleteAttributeEntry(self, index: int): ...
    def getAttributeEntries(self) -> NodegraphAPI.Parameter: ...
    def reorderAttributeEntries(self, oldPos: int, oldPosCount: int, newPos: int, newPosCount: int): ...
    def reorderAttributeEntry(self, oldPos: int, newPos: int): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.AnamorphDisplayOptions as AnamorphDisplayOptions
import CatalogAPI as CatalogAPI
import UI4.Util.DrawState as DrawState
import PyQt5.QtWidgets
import QT4Color as QT4Color
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
from QT4Widgets.PopdownLabel import PopdownLabel
from UI4.Tabs.Monitor.MenuUtils import AddDisplayVisualizationActionsToMenu as AddDisplayVisualizationActionsToMenu, AddImageChannelActionsToMenu as AddImageChannelActionsToMenu, AddPixelCheckControlsToMenu as AddPixelCheckControlsToMenu, AddViewControlsToMenu as AddViewControlsToMenu, AddZoomActionsToMenu as AddZoomActionsToMenu
from typing import ClassVar, Set, Tuple

class ColorAction(PyQt5.QtWidgets.QAction):
    def __init__(self, parent, text, valuePolicy, changeCB) -> None: ...
    def _ColorAction__colorChanged(self, vpEvent): ...
    def _ColorAction__triggered(self, *args): ...
    def setupEvents(self, enable): ...

class DisplayChannelsLabel(PopdownLabel):
    def __init__(self, monitorWidget, viewIndex, parent) -> None: ...
    def buildMenu(self, menu): ...
    def getDrawState(self): ...
    def updateState(self): ...

class DisplayExposureLabel(PyQt5.QtWidgets.QLabel):
    def __init__(self, monitorWidget, viewIndex, parent) -> None: ...
    def getDrawState(self): ...
    def getText(self): ...
    def isDefault(self): ...
    def mousePressEvent(self, ev): ...
    def updateState(self): ...

class DisplayScaleLabel(PyQt5.QtWidgets.QFrame):
    def __init__(self, monitorWidget: MonitorWidget, parent: QWidget) -> None: ...
    def _DisplayScaleLabel__onImageModeApplied(self): ...

class DisplayVisualizationLabel(PopdownLabel):
    def __init__(self, monitorWidget, viewIndex, parent) -> None: ...
    def buildMenu(self, menu): ...
    def getDrawState(self): ...
    def updateState(self): ...

class InputColorspaceLabel(PopdownLabel):
    def __init__(self, monitorWidget, viewIndex, parent) -> None: ...
    def buildMenu(self, menu): ...
    def getDrawState(self): ...
    def updateState(self): ...

class LayerSelectionLabel(PopdownLabel):
    LAYER_TOOLTIP_TEMPLATE: ClassVar[str] = ...
    def __init__(self, monitorWidget, viewIndex, parent, previousLayerShortcut, nextLayerShortcut, alternateDefaultLayerShortcut) -> None: ...
    def _LayerSelectionLabel__getLayerList(self): ...
    def _LayerSelectionLabel__offsetLayerSelection(self, offset): ...
    def alternateDefaultLayer(self): ...
    def buildMenu(self, menu): ...
    def getDrawState(self): ...
    def resetAlternateLayer(self): ...
    def selectNextLayer(self): ...
    def selectPreviousLayer(self): ...
    def updateState(self): ...

class MatteMultLabel(PopdownLabel):
    def __init__(self, monitorWidget, viewIndex, parent) -> None: ...
    def buildMenu(self, menu): ...
    def getDrawState(self): ...
    def updateState(self): ...

class SwipeModeLabel(PopdownLabel):
    def __init__(self, monitorWidget, *args) -> None: ...
    def _SwipeModeLabel__buildModeList(self, menu, actionGroup, selectedMode, modeList): ...
    def buildMenu(self, menu): ...
    def updateState(self): ...

class _DisplayScalePopdown(PopdownLabel):
    def __init__(self, monitorWidget: MonitorWidget, parent: QWidget) -> None: ...
    def buildMenu(self, menu): ...
    def layerStack_scaleChanged_CB(self, x, final): ...
    def updateState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.ParameterPolicy as ParameterPolicy
from UI4.FormMaster.BaseParameterPolicy import BaseParameterPolicy as BaseParameterPolicy
from typing import Set, Tuple

class GroupParameterPolicy(BaseParameterPolicy):
    def __init__(self, param, parent) -> None: ...
    def _GroupParameterPolicy__clearMissingCached(self): ...
    def _GroupParameterPolicy__getChildPolicyObject(self, childParam): ...
    def _GroupParameterPolicy__recursivelyUpdateHintsForPolicy(self, policy): ...
    def _childPolicyRenamed(self, childPolicy): ...
    def _freeze(self): ...
    def _handleHintChanged(self, args): ...
    def _handleNodeNameChanged(self, args: list[tuple[str, object, dict]]): ...
    def _handleTopologyChanged(self, args): ...
    def _thaw(self): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.AttributePolicy as AP
import NodegraphAPI as NodegraphAPI
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
import UI4.KatanaPrefs.PrefNames as PrefNames
import QT4FormWidgets as QT4FormWidgets
import UI4.FormMaster.States as States
import UI4.FormMaster.AttributePolicy
import Utils as Utils
import typing
from Callbacks.Callbacks import Callbacks as Callbacks
from UI4.KatanaPrefs.KatanaPrefsObject import Prefs as Prefs
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ArrayLiveAttributePolicy(LiveAttributePolicyMixin, UI4.FormMaster.AttributePolicy.ArrayAttributePolicy):
    @classmethod
    def Upgrade(cls, policy): ...
    def getArrayChild(self, index): ...
    def getChildPolicyObjects(self): ...
    def getValueState(self): ...

class GroupLiveAttributePolicy(LiveAttributePolicyMixin, UI4.FormMaster.AttributePolicy.GroupAttributePolicy):
    def _buildChildren(self, childDict, childNames): ...

class LiveAttributePolicyManager:
    class CacheEntry:
        def __init__(self) -> None: ...
    _LiveAttributePolicyManager__instance: ClassVar[None] = ...
    def __init__(self) -> None: ...
    @staticmethod
    def Instance(): ...
    def _LiveAttributePolicyManager__finalize3DNodeChanges(self, **kwargs): ...
    def _LiveAttributePolicyManager__on_pref_changed(self, eventType: str | None, eventID: typing.Hashable, prefKey: str, prefValue: object): ...
    def addValueChangedCallback(self, callback): ...
    def clearCache(self): ...
    def flush(self): ...
    def getCacheUndoName(self): ...
    def getValue(self, path, attr, index, getFunction): ...
    def isArrayLive(self, path, attr): ...
    def isLive(self, path, attr, index): ...
    def pause(self): ...
    def registerPolicy(self, policy, doRegister: bool = ...): ...
    def removeValueChangedCallback(self, callback): ...
    def resetTimer(self): ...
    def setValue(self, path, attr, index, value, setFunction: Incomplete | None = ..., keepAlive: Incomplete | None = ...): ...
    def unpause(self): ...
    def valueChangedCallback(self, policy, **kwargs): ...

class LiveAttributePolicyMixin:
    @classmethod
    def Upgrade(cls, policy): ...
    def _freeze(self): ...
    def _thaw(self): ...
    def addLiveCallback(self, callback): ...
    def delCallback(self, callback): ...
    def forwardValueChangedEvent(self, policy, **kwargs): ...
    def getManager(self): ...
    def valueChanged(self, value: bool = ..., final: bool = ..., stateChanged: bool = ..., topologyChanged: bool = ..., nameChanged: bool = ...): ...

class PageGroupLiveAttributePolicy(LiveAttributePolicyMixin, UI4.FormMaster.AttributePolicy.PageGroupAttributePolicy):
    def _buildChildren(self, childDict, childNames): ...

class ScalarLiveAttributePolicy(LiveAttributePolicyMixin, UI4.FormMaster.AttributePolicy.ScalarAttributePolicy):
    def __init__(self, *args, **kwargs) -> None: ...
    def _forwardToEditableParameterPolicy(self, name, *args, **kwargs): ...
    def _forwardToParameterPolicy(self, name, *args, **kwargs): ...
    def _getParameterPolicy(self, editable: bool = ...): ...
    def canHaveCurve(self, *args, **kwargs): ...
    def canHaveExpression(self): ...
    def forwardValueChangedEvent(self, policy, **kwargs): ...
    def getCurve(self): ...
    def getCurveAutoKey(self): ...
    def getCurveKey(self): ...
    def getCurveXml(self): ...
    def getExpression(self): ...
    def getExpressionError(self): ...
    def getExpressionNamespace(self): ...
    def getExpressionReference(self): ...
    def getValue(self): ...
    def getValueState(self): ...
    def hasFloatingCurve(self): ...
    def isCurveEnabled(self): ...
    def isCurveViewed(self): ...
    def isExpressionEnabled(self): ...
    def setCurve(self, curve): ...
    def setCurveAutoKey(self, state): ...
    def setCurveEnabled(self, state): ...
    def setCurveKey(self, state): ...
    def setCurveViewed(self, state): ...
    def setCurveXml(self, xml): ...
    def setExpression(self, expr): ...
    def setExpressionEnabled(self, flag): ...
    def setValue(self, value, final: bool = ...): ...

def ClearCache(): ...
def ConvertAttributePolicyToLive(ap): ...
def CreateProducerLiveGroupPolicy(): ...
def Flush(): ...
def Pause(): ...
def RegisterLiveAttributeChangedCallback(callback, register: bool = ...): ...
def Unpause(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import NodegraphAPI as NodegraphAPI
import NodegraphAPI_cmodule
import PyFnAttribute
import Utils as Utils
from _typeshed import Incomplete
from typing import Set, Tuple

class AttributeEditorNode3D(NodegraphAPI_cmodule.PythonGroupNode):
    def __init__(self) -> None: ...
    @classmethod
    def GetAttrType(cls, attr: PyFnAttribute.Attribute, default: Incomplete | None = ...) -> str | None: ...
    def _AttributeEditorNode3D__cleanupNodeLayout(self): ...
    def _AttributeEditorNode3D__createNewOverride(self, nodeType: str = ...): ...
    def _AttributeEditorNode3D__findExclusivityNode(self): ...
    def _AttributeEditorNode3D__getAllOverrides(self): ...
    def _AttributeEditorNode3D__isolateNode(self, node: NodegraphAPI.Node): ...
    def _AttributeEditorNode3D__setGroupOverride(self, path, attrName, time, attrData, makeEmptyGroup): ...
    def addParameterHints(self, attrName, inputDict): ...
    def canOverride(self, attrName): ...
    def copyOverride(self, fromLoc, toLoc, attrName, time: int = ...): ...
    def deleteOverride(self, loc, attrName, time: int = ...): ...
    def findOverride(self, path, attrName, time): ...
    def findOverrideParameter(self, path, attrName, time, index: Incomplete | None = ..., editable: bool = ...): ...
    def findTransform3D(self, path, time): ...
    def getExclusivity(self): ...
    def getOverrideContentsDict(self, time: int = ...): ...
    def ignoreOverride(self, loc, attrName, ignoreFlag): ...
    def isOverrideIgnored(self, loc, attrName): ...
    def moveOverride(self, fromLoc, toLoc, attrName, time: int = ...): ...
    def setExclusivity(self, celSelection): ...
    def setInteractiveTransform(self, path, absScale, absRotate, absTranslate, time): ...
    def setInteractiveTransformFlag(self, path, time): ...
    def setOverride(self, path, attrName, time, attrData, attrType: Incomplete | None = ..., attrTupleSize: Incomplete | None = ..., makeEmptyGroup: bool = ..., groupInherit: bool = ..., index: Incomplete | None = ..., hints: Incomplete | None = ...): ...
    def setPartialOverride(self, path, attrName, time, oldAttr, newIndex, newValue, groupInherit: bool = ...): ...

class AttributeEditorsDict(dict):
    def __init__(self, *args, **kwargs) -> None: ...
    def _AttributeEditorsDict__getAttributeEditor(self, attrName: Incomplete | None = ...): ...
    def getAttributeEditor(self, attrName: Incomplete | None = ...): ...
    def getNames(self): ...

def GetActiveAttributeEditorNode(scenegraphLocation, needAttrName: Incomplete | None = ..., needInteractiveTransform: bool = ...): ...
def GetAttributeEditorNode(): ...
def GetAttributeEditorsForAttr(attributeEditorAttr: PyFnAttribute.GroupAttribute) -> AttributeEditorsDict: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import DrawingModule as DrawingModule
import Utils.EventModule as EventModule
import UI4.Tabs.NodeGraphTab.Layers as Layers
import NodeGraphView as NodeGraphView
import NodegraphAPI as NodegraphAPI
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4GLLayerStack as QT4GLLayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import RenderingAPI as RenderingAPI
import UI4 as UI4
import Utils as Utils
import typing
import weakref
from Callbacks.Callbacks import Callbacks as Callbacks
from UI4.KatanaPrefs.KatanaPrefsObject import KatanaPrefs as KatanaPrefs
from UI4.Widgets.BaseNodeGraphLayerStack import BaseNodeGraphLayerStack
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

_NUM_VIEW_STATES: int

class NodegraphWidget(BaseNodeGraphLayerStack):
    _NodegraphWidget__connectPort: ClassVar[int] = ...
    _NodegraphWidget__connectReturn: ClassVar[int] = ...
    _NodegraphWidget__connectSend: ClassVar[int] = ...
    _NodegraphWidget__maxNGVViewScale: ClassVar[float] = ...
    _NodegraphWidget__minNGVViewScale: ClassVar[float] = ...
    _NodegraphWidget__networkMaterialAutoConnectableNodeTypeNames: ClassVar[list] = ...
    _NodegraphWidget__nodegraphWidgetList: ClassVar[weakref.WeakKeyDictionary] = ...
    inputSidebar: ClassVar[int] = ...
    outputSidebar: ClassVar[int] = ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget) -> None: ...
    def _NodegraphWidget__addInputPortWithParams(self, node: NodegraphAPI.Node, params): ...
    def _NodegraphWidget__addOutputPortWithParams(self, node: NodegraphAPI.Node, params): ...
    def _NodegraphWidget__autoConnectToTerminalNode(self, terminalNode, newNode): ...
    def _NodegraphWidget__buildLayers(self): ...
    def _NodegraphWidget__capNodeGraphViewScale(self, viewScale): ...
    def _NodegraphWidget__currentNodeViewChanged(self, _previousViewNode, currentViewNode): ...
    def _NodegraphWidget__editBackdropInNgvCallback(self, node: NodegraphAPI.Node, _mouseButton): ...
    def _NodegraphWidget__explodeSelectionReal(self): ...
    def _NodegraphWidget__gatherCurrentlySelectedBackdropsByZDepth(self): ...
    @staticmethod
    def _NodegraphWidget__getHitKey(a: tuple[str, dict]) -> int: ...
    def _NodegraphWidget__getTerminalFloatingNode(self): ...
    def _NodegraphWidget__idle_callback(self, *args, **kwargs): ...
    def _NodegraphWidget__isNodeAutoConnectable(self, terminalNode, newNode): ...
    def _NodegraphWidget__loadbegin_callback(self, *args, **kwargs): ...
    def _NodegraphWidget__loadend_callback(self, *args, **kwargs): ...
    def _NodegraphWidget__mergeSelectedNodesReal(self): ...
    def _NodegraphWidget__nodeDeleteHandler(self, eventType, eventID, node: NodegraphAPI.Node, *args, **kwargs): ...
    def _NodegraphWidget__nodeDeselectAllInNgvCallback(self): ...
    def _NodegraphWidget__nodeGroupEnteredInNgvCallback(self, node: NodegraphAPI.Node, _mouseButton): ...
    def _NodegraphWidget__nodeRenamedInNgvCallback(self, node: NodegraphAPI.Node, name): ...
    def _NodegraphWidget__nodeSelectedInNgvCallback(self, nodes: NodegraphAPI.Node | list[NodegraphAPI.Node], selected): ...
    def _NodegraphWidget__nodeSetBypassedInNgvCallback(self, node: NodegraphAPI.Node, bypassed): ...
    def _NodegraphWidget__nodeSetEditedInNgvCallback(self, node: NodegraphAPI.Node, edited): ...
    def _NodegraphWidget__nodeSetViewedInNgvCallback(self, node: NodegraphAPI.Node, viewed): ...
    def _NodegraphWidget__nodeStateRefreshRequestedCallback(self, node: NodegraphAPI.Node): ...
    def _NodegraphWidget__nodeToolTipRequestedInNgvCallback(self, node: NodegraphAPI.Node, coords): ...
    def _NodegraphWidget__nodeUpdatedInNgvCallback(self, node: NodegraphAPI.Node, noop): ...
    def _NodegraphWidget__nodeViewStateNextCallback(self, node: NodegraphAPI.Node): ...
    def _NodegraphWidget__offsetFloatingNodePositions(self, terminalNode, newNodes): ...
    def _NodegraphWidget__popupContextMenuInNgvCallback(self, node: NodegraphAPI.Node, _mouseButton): ...
    def _NodegraphWidget__popupPortMenuInNgvCallback(self, port: NodegraphAPI.Port, portCoords): ...
    def _NodegraphWidget__portConnectedInNgvCallback(self, portA, portB): ...
    def _NodegraphWidget__portDisconnectedInNgvCallback(self, portA, portB): ...
    def _NodegraphWidget__portRenamedInNgvCallback(self, port: NodegraphAPI.Port, newPortName): ...
    def _NodegraphWidget__prefChanged_CB(self, eventType, eventID, *args, **kwargs): ...
    def _NodegraphWidget__reconnectAfterCollapse(self, nodes): ...
    def _NodegraphWidget__sidebarUpdatedInNgvCallback(self, groupNode, sidebar): ...
    def _NodegraphWidget__smoothstep(self, a, b, x): ...
    def _NodegraphWidget__startDragInNgvCallback(self, obj, _mouseButton): ...
    def _NodegraphWidget__toggleNodesInputConnectionsVisibilityHandler(self, *_, **kwargs): ...
    def _NodegraphWidget__toggleSelectedNodesInputConnectionsVisibilityHandler(self, *_): ...
    def _NodegraphWidget__toolTipRequestedInNgvCallback(self, port: NodegraphAPI.Port, portCoords): ...
    def _NodegraphWidget__updateDimmingMode(self, mode): ...
    def alignSelection(self): ...
    def autoPlace(self, nodes: Sequence[NodegraphAPI.Node]) -> bool: ...
    def canCollapseSelectionToStack(self): ...
    def cleanup(self): ...
    def clearTransientLayers(self): ...
    def collapseSelectedToStack(self): ...
    def collapseSelection(self): ...
    def createNodeType(self, nodeType, shouldFloat: bool = ..., maskInputPreferred: bool = ..., autoPlaceAllowed: bool = ...): ...
    def createStandardOrRendererSpecificNodes(self, value): ...
    def enableFloatingLayer(self): ...
    def event(self, e): ...
    def explodeSelection(self): ...
    def extractNodes(self, nodes): ...
    def fitBackdropNode(self): ...
    def fitSticky(self): ...
    def floatNodes(self, nodes: Sequence[NodegraphAPI.Node]): ...
    def focusInEvent(self, event: QFocusEvent): ...
    def focusOutEvent(self, event: QFocusEvent): ...
    def frameNodes(self, nodes, zoom: bool = ...): ...
    @staticmethod
    def getAllNodeGraphWidgets(): ...
    def getCurrentNodeView(self): ...
    def getGraphInteraction(self): ...
    def getGroupNodeUnderMouse(self): ...
    def getNetworkMaterialContextNodeNames(self) -> list[str]: ...
    def getNetworkMaterialNodeTypeNames(self) -> list[str]: ...
    def getNodeGraphContext(self) -> str: ...
    def getPointAdjustedToGroupNodeSpace(self, groupNode, point): ...
    def getPopupMenuClickLocation(self): ...
    def getPseudoTopLevelWidget(self): ...
    def getRecentNode(self): ...
    def getRendererContext(self, node: NodegraphAPI.Node) -> str: ...
    def getValidNodeTypesForCurrentContext(self): ...
    def getValidNodeTypesForCurrentRenderer(self): ...
    def getWorldBounds(self): ...
    def hasGroupNodeFocusChanged(self): ...
    def hitTestBox(self, p0, p1, viewNode: Incomplete | None = ...): ...
    def hitTestFirstHitForType(self, p, type): ...
    def hitTestFirstHitForTypeWithOptions(self, p, hitType, *args): ...
    def hitTestForType(self, p, type, excludeNodes: Incomplete | None = ...): ...
    def hitTestForTypeStartsWith(self, p, type): ...
    def hitTestForTypeWithOptions(self, p, hitType, *args): ...
    def hitTestNodeButtons(self, p): ...
    def hitTestPoint(self, p): ...
    def idleUpdate(self): ...
    def isolateNode(self, node: NodegraphAPI.Node): ...
    def mergeSelectedNodes(self): ...
    def nodeMenu(self): ...
    def paintGL(self): ...
    def placeNode(self, node: NodegraphAPI.Node, shouldFloat: bool = ..., maskInputPreferred: bool = ..., autoPlaceAllowed: bool = ...): ...
    def placeNodes(self, nodes: Sequence[NodegraphAPI.Node], shouldFloat: bool = ..., autoPlaceAllowed: bool = ...): ...
    def removeFloatingNode(self): ...
    def removeFloatingNodes(self): ...
    def resizeEvent(self, event: PyQt5.QtGui.QResizeEvent): ...
    def sendSelectedBackdropNodesToBack(self): ...
    def sendSelectedBackdropNodesToFront(self): ...
    def setCurrentNodeView(self, node: NodegraphAPI.Node): ...
    def setEyePoint(self, x): ...
    def setFloatingLayerBasePosition(self, pos): ...
    def setGraphInteraction(self, flag): ...
    def setGroupNodeAsRoot(self, groupNode): ...
    def setNGVShapeAttrs(self, node: NodegraphAPI.Node, attrDict): ...
    def setNodeGraphContext(self, context): ...
    def setPopupMenuClickLocation(self, x, y): ...
    def setViewScale(self, x, final: bool = ...): ...
    def shouldAutoScroll(self): ...
    def showEvent(self, event: PyQt5.QtGui.QShowEvent): ...
    def showLayeredMenu(self, menu): ...
    def toggleTeleports(self): ...
    @classmethod
    def updateAllNodegraphWidgets(cls): ...
    def updateOtherNodegraphsInSameNode(self): ...

def _OnNodeCreatedCallback(node: typing.Optional[NodegraphAPI.Node] = ..., nodeType: Incomplete | None = ..., **_kwargs): ...
def _SendBackdropNodesToBack(nodes: list) -> bool: ...
def _SendBackdropNodesToFront(nodes: list, undo: bool = ...) -> bool: ...
def __nodegraphPrefsCallback(*args, **kwargs): ...
def __nodegraphRedrawCallback(args): ...
def _getZDepthAttr(node: NodegraphAPI.Node): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from UI4.Widgets.PanelScrollArea import PanelScrollArea
from typing import Set, Tuple

class ProjectSettingsPanel(PanelScrollArea):
    def __init__(self, *args) -> None: ...
    def _ProjectSettingsPanel__collapsedEventHandler(self, args): ...
    def _ProjectSettingsPanel__rebuild(self): ...
    def highlightParameter(self, path): ...
    def showFindPopup(self): ...

class ProjectSettingsTab(BaseTab):
    def __init__(self, parent) -> None: ...
    def _ProjectSettingsTab__findSettingActionCallback(self): ...
    def highlightParameter(self, path): ...
    @staticmethod
    def registerKeyboardShortcuts(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFCurve as PyFCurve
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QTFCurve.FCurveListView import FCurveListView as FCurveListView
from QTFCurve.FCurveValueEdit import FCurveValueEdit as FCurveValueEdit
from _typeshed import Incomplete
from typing import Set, Tuple

class FCurveGraphValueListWidget(PyQt5.QtWidgets.QWidget):
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def addCustomColumn(self, name, defaultValue, valueList: list = ...): ...
    def getCurves(self) -> Container: ...
    def getGraphView(self) -> GraphWidget: ...
    def getListView(self) -> FCurveListView: ...
    def getValueEdit(self) -> FCurveValueEdit: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import Set, Tuple

def axisAngleToQuaternion(axis, angle): ...
def crossProduct3(a, b): ...
def decomposeMatrix4x4(m): ...
def dotProduct3(a, b): ...
def explodeMatrix4x4(m): ...
def identity(): ...
def invert(m): ...
def magnitude3(a): ...
def multiply(a: None, b: None): ...
def multiplyv3(m: None, v: None): ...
def multiplyv4(m: None, v: None): ...
def normalize3(a): ...
def quaternionToRotationMatrix(quat): ...
def scaleMatrix(x, y, z): ...
def transformPoint4ByMatrix4x4(p, m): ...
def translationMatrix(x, y, z): ...
def transpose4x4(m): ...
def xRotationMatrix(x): ...
def xformPointByMatrix(p, m): ...
def yRotationMatrix(y): ...
def zRotationMatrix(z): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import UI4.FormMaster.HintsDelegate as HintsDelegate
import Naming as Naming
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import UI4.FormMaster.Pages as Pages
import PyFnGeolibProducers
import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4.FormMaster.States as States
import Utils as Utils
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ArrayAttributePolicy(BaseAttributePolicy):
    def __init__(self, attr, typeName, name, parent, stateSource: Incomplete | None = ...) -> None: ...
    def _ArrayAttributePolicy__getChildPolicyObject(self, index): ...
    def _equivalent(self, other): ...
    def _setAttr(self, attr): ...
    def getArrayChild(self, index): ...
    def getArraySize(self): ...
    def getChildPolicyObjects(self): ...
    def getNumChildren(self): ...
    def getType(self): ...
    def setValueState(self, state): ...

class AttributeStateSource(PyQt5.QtCore.QObject):
    _AttributeStateSource__activeStateMappings: ClassVar[dict] = ...
    _AttributeStateSource__stateMappings: ClassVar[dict] = ...
    showAttributeHistory: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showAttributeInheritance: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, producer: PyFnGeolibProducers.GeometryProducer, path, hintDict, scopeDict, stripLen: int = ...) -> None: ...
    def _AttributeStateSource__getKey(self, key): ...
    def _transferState(self, rhs): ...
    def _triggerAttributeHistory(self, policy): ...
    def _triggerAttributeInheritance(self, policy): ...
    def getActiveAttributeEditor(self, policy): ...
    def getAttrPath(self, policy): ...
    def getAvailableStates(self, policy): ...
    def getHints(self, path): ...
    def getPath(self): ...
    def getProducer(self): ...
    def getState(self, path): ...
    def isLocked(self, policy): ...
    def setAttributeEditorsDict(self, editors): ...
    def setLocked(self, state): ...

class BaseAttributePolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    def __init__(self, attr, name, parent, stateSource: Incomplete | None = ...) -> None: ...
    def _BaseAttributePolicy__attributeHistoryTrigger(self, checked): ...
    def _BaseAttributePolicy__attributeInheritanceTrigger(self, checked): ...
    def _appendToStateMenu(self, menu: PyQt5.QtWidgets.QMenu) -> bool: ...
    def _equivalent(self, other): ...
    def _getAttr(self): ...
    def _getFullAttrName(self): ...
    def _setAttr(self, attr): ...
    def getAttrPath(self): ...
    def getAvailableStates(self): ...
    def getIndex(self): ...
    def getMimeData(self) -> PyQt5.QtCore.QMimeData: ...
    def getName(self): ...
    def getPath(self): ...
    def getScenegraphPath(self): ...
    def getStateAppearance(self, state): ...
    def getStateSource(self): ...
    def getValueAsAttr(self): ...
    def getValueState(self): ...
    def getWidgetHints(self): ...
    def isLocked(self): ...
    def shouldDisplayState(self): ...

class GroupAttributePolicy(BaseAttributePolicy):
    def __init__(self, attr, name, parent, stateSource: Incomplete | None = ...) -> None: ...
    def _GroupAttributePolicy__buildChildren(self): ...
    def _GroupAttributePolicy__getChildPageHintCallback(self, childName): ...
    def _buildChildren(self, childDict, childNames): ...
    def _setAttr(self, attr): ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getIgnorePages(self): ...
    def getNumChildren(self): ...
    def getType(self): ...
    def setIgnorePages(self, ignorePages): ...

class PageGroupAttributePolicy(GroupAttributePolicy):
    def __init__(self, attr, name, parent, childNames, childPages, stateSource: Incomplete | None = ..., pagePath: str = ...) -> None: ...
    def _buildChildren(self, childDict, childNames): ...
    def _getFullAttrName(self): ...
    def getValueState(self): ...
    def getWidgetHints(self): ...
    def shouldDisplayState(self): ...

class ScalarAttributePolicy(BaseAttributePolicy):
    def __init__(self, attr, index, typeName, name, parent, stateSource: Incomplete | None = ...) -> None: ...
    def _ScalarAttributePolicy__buildEmptyOverride(self): ...
    def _setAttr(self, attr): ...
    def getIndex(self): ...
    def getType(self): ...
    def getValue(self): ...
    def getValueState(self): ...
    def getWidgetHints(self): ...
    def setValue(self, value, final: bool = ...): ...
    def setValueState(self, state): ...

def CreateAttributePolicy(parentPolicy, name, attr, stateSource: Incomplete | None = ...): ...
def CreateProducerGroupPolicy(producer: PyFnGeolibProducers.GeometryProducer, attributes: Incomplete | None = ..., excludeAttributes: Incomplete | None = ..., ignorePages: bool = ..., deepExclude: Incomplete | None = ...): ...
